@@ -55,6 +55,8 @@ typedef struct  pixmap_sb_view
 
 	GC  gc ;
 
+	unsigned int  depth ;
+
 	char *  dir ;
 	int  is_transparent ;
 	unsigned int  width ;
@@ -109,8 +111,12 @@ typedef struct  pixmap_sb_view
 
 /* --- static functions --- */
 
-/* declared in x_imagelib.h */
-int  x_imagelib_load_file( Display *  display, char *  path, u_int32_t **  cardinal,
+/* XXX Hack (declared in x_display.h) */
+typedef struct x_display_t *  x_display_ptr_t ;
+x_display_ptr_t *  x_get_opened_displays( u_int *  num) ;
+
+/* XXX Hack (declared in x_imagelib.h) */
+int  x_imagelib_load_file( x_display_ptr_t  disp, char *  path, u_int32_t **  cardinal,
 	Pixmap *  pixmap, Pixmap *  mask, int *  width, int *  height) ;
 
 static void
@@ -126,6 +132,9 @@ load_image(
 {
 	char *  path ;
 	int  len ;
+	x_display_ptr_t *  disps ;
+	u_int  ndisps ;
+	u_int  count ;
 
 	if( height == NULL || width == NULL)
 	{
@@ -136,7 +145,31 @@ load_image(
 	path = malloc( sizeof( char) * ( len + 1)) ;
 	sprintf( path , "%s/%s.png" , dir , file);
 
-	if( ! x_imagelib_load_file( display , path , NULL , pixmap , mask ,
+	disps = x_get_opened_displays( &ndisps) ;
+	count = 0 ;
+	while( 1)
+	{
+		/*
+		 * XXX Hack
+		 * (*disps) == (x_display_t*) == (Display**)
+		 * (Display* is the first member of x_display_t.)
+		 */
+		if( *((Display**)(*disps)) == display)
+		{
+			break ;
+		}
+
+		if( ++count >= ndisps)
+		{
+			break ;
+		}
+		else
+		{
+			disps ++ ;
+		}
+	}
+	
+	if( ! x_imagelib_load_file( *disps , path , NULL , pixmap , mask ,
 				width , height))
 	{
 #ifdef __DEBUG
@@ -179,8 +212,7 @@ create_bg_cache(
 		return ;
 	}
 
-	ps->bg_cache = XCreatePixmap( d , win , ps->width , bg_h ,
-				DefaultDepth( d , ps->view.screen)) ;
+	ps->bg_cache = XCreatePixmap( d , win , ps->width , bg_h , ps->depth) ;
 
 	if( ps->bg_body_h && ps->bg_body)
 	{
@@ -252,7 +284,7 @@ resize_slider(
 	free_pixmap( d , ps->slider_tiled_cache) ;
 
 	ps->slider_tiled_cache = XCreatePixmap( d , win , ps->slider_width ,
-			body_height , DefaultDepth( d , ps->view.screen)) ;
+					body_height , ps->depth) ;
 
 	if( ps->slider_body_h && ps->slider_body)
 	{
@@ -348,6 +380,7 @@ realized(
 {
 	pixmap_sb_view_t *  ps ;
 	XGCValues  gc_value ;
+	XWindowAttributes  attr ;
 
 	ps = (pixmap_sb_view_t*) view ;
 
@@ -363,6 +396,9 @@ realized(
 
 	ps->gc = XCreateGC( display , window , GCForeground | GCBackground | GCGraphicsExposures , &gc_value) ;
 
+	XGetWindowAttributes( view->display , view->window , &attr) ;
+	ps->depth = attr.depth ;
+
 	/*
 	 * load background images (separated three parts: top, body and bottom.)
 	 */
