@@ -1,16 +1,31 @@
 /*
- *	$Id: kik_debug.h,v 1.4 2002/02/02 09:20:27 arakiken Exp $
+ *	$Id$
  */
 
 #ifndef  __KIK_DEBUG_H__
 #define  __KIK_DEBUG_H__
 
 
-#include  "kik_def.h"
+#include  "kik_config.h"
 
 
+#define  KIK_INT_TO_STR(i)  _KIK_INT_TO_STR(i)
+#define  _KIK_INT_TO_STR(i) #i
+
 /* for kik_{debug|warn}_printf */
+#if  0
+
+#ifdef  CONCATABLE_FUNCTION
 #define  KIK_DEBUG_TAG  "[" __FUNCTION__ "()]"
+#else
+#define  KIK_DEBUG_TAG  "[" __FILE__ "]"
+#endif
+
+#else
+
+#define  KIK_DEBUG_TAG  "[" __FILE__ ":" KIK_INT_TO_STR(__LINE__) "]"
+
+#endif
 
 
 int  kik_debug_printf( const char *  format , ...) ;
