@@ -6,16 +6,29 @@
 
 /* --- global functions --- */
 
-/*
- * slave_name memory must be freed by a caller.
- */
 pid_t
 kik_pty_fork(
 	int *  master ,
-	int *  slave ,
-	char **  slave_name
+	int *  slave
 	)
 {
 	/* do nothing. */
 	return  0 ;
 }
+
+int
+kik_pty_helper_close(
+	int  pty
+	)
+{
+	return  0 ;
+}
+
+void
+kik_pty_helper_set_flag(
+	int  lastlog ,
+	int  utmp ,
+	int  wtmp
+	)
+{
+}
