@@ -280,10 +280,17 @@ ml_char_combine(
 		return  0 ;
 	}
 
+	/*
+	 * This check should be excluded, because characters whose is_comb flag
+	 * (combining property of mkf) is NULL can be combined
+	 * if ml_is_arabic_combining(them) returns non-NULL.
+	 */
+#if  0
 	if( ! is_comb)
 	{
 		return  0 ;
 	}
+#endif
 
 	if( IS_SINGLE_CH(ch->u.ch.attr))
 	{
@@ -338,7 +345,7 @@ ml_char_combine(
 			return  0 ;
 		}
 
-#if !defined(__GLIBC__)
+	#if !defined(__GLIBC__)
 		if( sizeof( multi_ch) >= 8 && ((long)( multi_ch) & 0x1UL) != 0)
 		{
 			kik_msg_printf( "Your malloc() doesn't return 2 bits aligned address."
@@ -346,7 +353,7 @@ ml_char_combine(
 
 			return  0 ;
 		}
-#endif
+	#endif
 
 		SET_COMB_TRAILING( multi_ch[comb_size].u.ch.attr) ;
 		ml_char_init( multi_ch + comb_size + 1) ;
@@ -950,7 +957,8 @@ ml_sp_ch(void)
 	if( sp_ch.u.ch.attr == 0)
 	{
 		ml_char_init( &sp_ch) ;
-		ml_char_set( &sp_ch , (u_char *)" " , 1 , US_ASCII , 0 , 0 , ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
+		ml_char_set( &sp_ch , (u_char *)" " , 1 , US_ASCII , 0 , 0 ,
+			ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
 	}
 
 	return  &sp_ch ;
@@ -964,8 +972,8 @@ ml_nl_ch(void)
 	if( nl_ch.u.ch.attr == 0)
 	{
 		ml_char_init( &nl_ch) ;
-		ml_char_set( &nl_ch , (u_char *)"\n" , 1 , US_ASCII , 0 , 0 , ML_FG_COLOR , ML_BG_COLOR ,
-			0 , 0) ;
+		ml_char_set( &nl_ch , (u_char *)"\n" , 1 , US_ASCII , 0 , 0 ,
+			ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
 	}
 
 	return  &nl_ch ;
