@@ -27,7 +27,6 @@ typedef struct  ml_pty
 {
 	int  master ;		/* master pty fd */
 	int  slave ;
-	char *  slave_name ;
 	pid_t  child_pid ;
 
 #ifdef  USE_UTMP
@@ -44,14 +43,13 @@ typedef struct  ml_pty
 } ml_pty_t ;
 
 
-
 /* --- global functions --- */
 
 ml_pty_t *
 ml_pty_new(
-	char *  cmd_path ,
-	char **  cmd_argv ,
-	char **  env ,
+	char *  cmd_path ,	/* can be NULL */
+	char **  cmd_argv ,	/* can be NULL(only if cmd_path is NULL) */
+	char **  env ,		/* can be NULL */
 	char *  host ,
 	u_int  cols ,
 	u_int  rows
@@ -65,7 +63,7 @@ ml_pty_new(
 		return  NULL ;
 	}
 
-	pid = kik_pty_fork( &pty->master , &pty->slave , &pty->slave_name) ;
+	pid = kik_pty_fork( &pty->master , &pty->slave) ;
 
 	if( pid == -1)
 	{
@@ -76,23 +74,44 @@ ml_pty_new(
 	{
 		/* child process */
 
-		/*
-		 * setting environmental variables.
-		 */
-		while( *env)
-		{
-			/* an argument string of putenv() must be allocated memory.(see SUSV2) */
-			putenv( strdup( *env)) ;
-
-			env ++ ;
-		}
-
 		/* reset signals and spin off the command interpreter */
 		signal(SIGINT, SIG_DFL) ;
 		signal(SIGQUIT, SIG_DFL) ;
 		signal(SIGCHLD, SIG_DFL) ;
 		signal(SIGPIPE, SIG_DFL) ;
 
+		if( ! cmd_path)
+		{
+			pty->child_pid = 0 ;
+		#ifdef  USE_UTMP
+			pty->utmp = NULL ;
+		#endif
+			pty->buf = NULL ;
+			pty->left = 0 ;
+			pty->size = 0 ;
+
+			pty->pty_listener = NULL ;
+			
+			return  pty ;
+		}
+		
+		/*
+		 * setting environmental variables.
+		 */
+		if( env)
+		{
+			while( *env)
+			{
+				/*
+				 * an argument string of putenv() must be allocated memory.
+				 * (see SUSV2)
+				 */
+				putenv( strdup( *env)) ;
+
+				env ++ ;
+			}
+		}
+
 	#if  0
 		/*
 		 * XXX is this necessary ?
@@ -112,8 +131,6 @@ ml_pty_new(
 			#ifdef  DEBUG
 				kik_warn_printf( KIK_DEBUG_TAG " execve(%s) failed.\n" , cmd_path) ;
 			#endif
-			
-				exit(1) ;
 			}
 		}
 		else
@@ -132,7 +149,7 @@ ml_pty_new(
 	/* parent process */
 
 #ifdef  USE_UTMP
-	if( ( pty->utmp = kik_utmp_new( pty->slave_name , host , pty->master)) == NULL)
+	if( ( pty->utmp = kik_utmp_new( ttyname( pty->slave) , host , pty->master)) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG "utmp failed.\n") ;
@@ -178,9 +195,10 @@ ml_pty_delete(
 	kik_debug_printf( "%d fd is closed\n" , pty->master) ;
 #endif
 
+	kik_pty_helper_close( pty->master) ;
+
 	close( pty->master) ;
 	close( pty->slave) ;
-	free( pty->slave_name) ;
 		
 	free( pty) ;
 
@@ -451,5 +469,5 @@ ml_pty_get_slave_name(
 	ml_pty_t *  pty
 	)
 {
-	return  pty->slave_name ;
+	return  ttyname( pty->slave) ;
 }
