@@ -60,7 +60,6 @@ typedef struct  ml_screen
 	int8_t  is_backscrolling ;
 
 	int8_t  use_dynamic_comb ;	/* public */
-	int8_t  use_bce ;
 	int8_t  is_cursor_visible ;
 
 } ml_screen_t ;
@@ -68,7 +67,7 @@ typedef struct  ml_screen
 
 int  ml_set_word_separators( char *  seps) ;
 
-int  ml_free_word_separators(void) ;
+#define  ml_free_word_separators()  ml_set_word_separators(NULL)
 
 
 ml_screen_t *  ml_screen_new( u_int  cols , u_int  rows , u_int  tab_size ,
@@ -80,10 +79,6 @@ int  ml_screen_set_listener( ml_screen_t *  screen , ml_screen_event_listener_t
 
 int  ml_screen_resize( ml_screen_t *  screen , u_int  cols , u_int  rows) ;
 
-int  ml_screen_use_bce( ml_screen_t *  screen) ;
-
-int  ml_screen_unuse_bce( ml_screen_t *  screen) ;
-
 int  ml_screen_set_bce_fg_color( ml_screen_t *  screen , ml_color_t  fg_color) ;
 
 int  ml_screen_set_bce_bg_color( ml_screen_t *  screen , ml_color_t  bg_color) ;
