@@ -28,6 +28,7 @@ ml_term_new(
 	int  use_char_combining ,
 	int  use_multi_col_char ,
 	int  use_bidi ,
+	ml_bidi_mode_t  bidi_mode ,
 	int  use_bce ,
 	int  use_dynamic_comb ,
 	ml_bs_mode_t  bs_mode ,
@@ -49,7 +50,8 @@ ml_term_new(
 	term->pty = NULL ;
 	term->pty_listener = NULL ;
 
-	if( ( term->screen = ml_screen_new( cols , rows , tab_size , log_size , use_bce , bs_mode)) == NULL)
+	if( ( term->screen = ml_screen_new( cols , rows ,
+				tab_size , log_size , use_bce , bs_mode)) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " ml_screen_new failed.\n") ;
@@ -58,8 +60,8 @@ ml_term_new(
 		goto  error ;
 	}
 
-	if( ( term->parser = ml_vt100_parser_new( term->screen , encoding , policy ,
-					col_size_a , use_char_combining , use_multi_col_char)) == NULL)
+	if( ( term->parser = ml_vt100_parser_new( term->screen , encoding , policy , col_size_a ,
+				use_char_combining , use_multi_col_char)) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " ml_vt100_parser_new failed.\n") ;
@@ -81,6 +83,7 @@ ml_term_new(
 	term->iscii_lang = NULL ;
 	term->iscii_lang_type = iscii_lang_type ;
 	term->vertical_mode = vertical_mode ;
+	term->bidi_mode = bidi_mode ;
 	term->use_bidi = use_bidi ;
 	term->use_dynamic_comb = use_dynamic_comb ;
 
@@ -147,6 +150,23 @@ ml_term_delete(
 	return  1 ;
 }
 
+int
+ml_term_zombie(
+	ml_term_t *  term
+	)
+{
+	ml_pty_ptr_t  pty ;
+
+	pty = term->pty ;
+
+	/* Should be NULL because ml_pty_delete calls term->pty_listener->closed. */
+	term->pty = NULL ;
+
+	ml_pty_delete( pty) ;
+
+	return  1 ;
+}
+
 int
 ml_term_open_pty(
 	ml_term_t *  term ,
@@ -198,7 +218,7 @@ ml_term_attach(
 {
   	if( term->is_attached)
         {
-          	/* already attached or pty not opened. */
+          	/* already attached */
           	return  0 ;
         }
 
@@ -208,7 +228,7 @@ ml_term_attach(
 	
 	if( term->pty)
 	{
-		ml_pty_set_listener( term->pty, pty_listener) ;
+		ml_pty_set_listener( term->pty , pty_listener) ;
 	}
 	else
 	{
@@ -227,7 +247,7 @@ ml_term_detach(
 {
   	if( ! term->is_attached)
         {
-          	/* already detached or pty not opened. */
+          	/* already detached. */
           	return  0 ;
         }
 
@@ -237,7 +257,7 @@ ml_term_detach(
 
 	if( term->pty)
 	{
-		ml_pty_set_listener( term->pty, NULL) ;
+		ml_pty_set_listener( term->pty , NULL) ;
 	}
 	else
 	{
@@ -857,9 +877,9 @@ ml_term_update_special_visual(
 			}
 
 		#if  1
-			if( ( logvis = ml_logvis_bidi_new( 0)) == NULL)
+			if( ( logvis = ml_logvis_bidi_new( 0 , term->bidi_mode)) == NULL)
 		#else
-			if( ( logvis = ml_logvis_bidi_new( 1)) == NULL)
+			if( ( logvis = ml_logvis_bidi_new( 1 , term->bidi_mode)) == NULL)
 		#endif
 			{
 			#ifdef  DEBUG
@@ -1063,8 +1083,8 @@ ml_term_get_word_region(
 	int  base_row
 	)
 {
-	return  ml_screen_get_word_region( term->screen , beg_char_index , beg_row , end_char_index ,
-			end_row , base_char_index , base_row) ;
+	return  ml_screen_get_word_region( term->screen , beg_char_index , beg_row ,
+			end_char_index , end_row , base_char_index , base_row) ;
 }
 
 int
