@@ -37,6 +37,7 @@ typedef struct ml_term
 	ml_iscii_lang_type_t  iscii_lang_type ;
 	ml_iscii_lang_t  iscii_lang ;
 	ml_vertical_mode_t  vertical_mode ;
+	ml_bidi_mode_t  bidi_mode ;
 	
 	int8_t  use_bidi ;
 	int8_t  use_dynamic_comb ;
@@ -61,11 +62,13 @@ typedef struct ml_term
 ml_term_t *  ml_term_new( u_int  cols , u_int  rows , u_int  tab_size , u_int  log_size ,
 	ml_char_encoding_t  encoding , int  is_auto_encoding , ml_unicode_font_policy_t  policy ,
 	int  col_size_a , int  use_char_combining , int  use_multi_col_char , int  use_bidi ,
-	int  use_bce , int  use_dynamic_comb , ml_bs_mode_t  bs_mode ,
+	ml_bidi_mode_t  bidi_mode , int  use_bce , int  use_dynamic_comb , ml_bs_mode_t  bs_mode ,
 	ml_vertical_mode_t  vertical_mode , ml_iscii_lang_type_t  iscii_lang_type) ;
 
 int  ml_term_delete( ml_term_t *  term) ;
 
+int  ml_term_zombie( ml_term_t *  term) ;
+
 int  ml_term_open_pty( ml_term_t *  term , char *  cmd_path , char **  argv , char **  env , char *  host) ;
 
 int  ml_term_attach( ml_term_t *  term , ml_xterm_event_listener_t *  xterm_listener ,
