@@ -16,7 +16,7 @@ int  ml_term_manager_final( void) ;
 ml_term_t *  ml_create_term( u_int  cols , u_int  rows , u_int  tab_size , u_int  log_size ,
 	ml_char_encoding_t  encoding , int  is_auto_encoding , ml_unicode_font_policy_t  policy ,
 	int  col_size_a , int  use_char_combining , int  use_multi_col_char , int  use_bidi ,
-	int  use_bce , int  use_dynamic_comb , ml_bs_mode_t  bs_mode ,
+	ml_bidi_mode_t  bidi_mode , int  use_bce , int  use_dynamic_comb , ml_bs_mode_t  bs_mode ,
 	ml_vertical_mode_t  vertical_mode , ml_iscii_lang_type_t  iscii_lang_type) ;
 
 int  ml_destroy_term( ml_term_t *  term) ;
@@ -35,5 +35,7 @@ int  ml_close_dead_terms(void) ;
 
 char *  ml_get_pty_list(void) ;
 
+void  ml_term_manager_enable_zombie_pty( int  bool) ;
+
 
 #endif
