@@ -148,7 +148,7 @@ mc_bgtype_config_widget_new(void)
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), radio, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), wall_picture, TRUE, TRUE, 0);
-#if !defined(USE_IMLIB) && !defined(USE_GDK_PIXBUF)
+#ifndef  USE_EXT_IMAGELIB
     gtk_widget_set_sensitive(radio, 0);
 #endif
 
