@@ -27,14 +27,21 @@ typedef struct  x_display
 	/*
 	 * Public(read only)
 	 */
-	Display *  display ;
+	Display *  display ;	/* Don't change position, which pixmap_engine depends on. */
 	int  screen ;		/* DefaultScreen */
 	char *  name ;
+	Window  my_window ;	/* DefaultRootWindow */
+
+#ifndef  USE_WIN32GUI
+	/* Only one visual, colormap or depth is permitted per display. */
+	Visual *  visual ;
+	Colormap  colormap ;
+#endif
+	u_int  depth ;
 
 	/*
 	 * Private
 	 */
-	Window  my_window ;	/* DefaultRootWindow */
 	x_gc_t *  gc ;
 
 	x_window_ptr_t *  roots ;
@@ -51,7 +58,7 @@ typedef struct  x_display
 } x_display_t ;
 
 
-x_display_t *  x_display_open( char *  disp_name) ;
+x_display_t *  x_display_open( char *  disp_name , u_int  depth) ;
 
 int  x_display_close( x_display_t *  disp) ;
 
@@ -62,7 +69,7 @@ x_display_t **  x_get_opened_displays( u_int *  num) ;
 int  x_display_fd( x_display_t *  disp) ;
 
 int  x_display_show_root( x_display_t *  disp , x_window_ptr_t  root ,
-	int  x , int  y , int  hint , char *  app_name) ;
+	int  x , int  y , int  hint , char *  app_name , Window  parent_window) ;
 
 int  x_display_remove_root( x_display_t *  disp , x_window_ptr_t  root) ;
 
