@@ -17,6 +17,8 @@
 
 #define  FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p) \
 	for( (font_encoding_p) = &csinfo->encoding_names[0] ; *(font_encoding_p) ; (font_encoding_p) ++)
+#define  DIVIDE_ROUNDING(a,b)  ( ((int)((a)*10 + (b)*5)) / ((int)((b)*10)) )
+#define  DIVIDE_ROUNDINGUP(a,b) ( ((int)((a)*10 + (b)*10 - 1)) / ((int)((b)*10)) )
 
 #if  0
 #define  __DEBUG
@@ -132,6 +134,9 @@ static cs_info_t  cs_info_table[] =
 } ;
 
 static int  compose_dec_special_font ;
+#ifdef  USE_TYPE_XFT
+static char *  xft_size_type = XFT_PIXEL_SIZE ;
+#endif
 
 
 /* --- static functions --- */
@@ -454,12 +459,12 @@ next_char:
 static u_int
 get_xft_col_width(
 	x_font_t *  font ,
-	const char *  family ,
-	int  weight ,
-	int  slant ,
-	double  fontsize
+	double  fontsize_d ,
+	u_int  percent ,
+	u_int  letter_space
 	)
 {
+#if  0
 	XftFont *  xfont ;
 
 	/*
@@ -468,7 +473,7 @@ get_xft_col_width(
 	 */
 	if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
 		XFT_FAMILY , XftTypeString , family ,
-		XFT_PIXEL_SIZE , XftTypeDouble , fontsize ,
+		xft_size_type , XftTypeDouble , fontsize ,
 		XFT_WEIGHT , XftTypeInteger , weight ,
 		XFT_SLANT , XftTypeInteger , slant ,
 		XFT_ENCODING , XftTypeString , "iso8859-1" ,
@@ -476,7 +481,8 @@ get_xft_col_width(
 	{
 		u_int  w_width ;
 
-		w_width = xft_calculate_char_width( font->display , xfont , "W" , 1) ;
+		w_width = xft_calculate_char_width( font->display , xfont , "W" , 1)
+				+ letter_space ;
 
 		XftFontClose( font->display , xfont) ;
 
@@ -489,9 +495,34 @@ get_xft_col_width(
 			return  w_width ;
 		}
 	}
+#endif
 
-	/* XXX this may be inaccurate. */
-	return  fontsize / 2 ;
+	if( percent > 0)
+	{
+		return  DIVIDE_ROUNDING(fontsize_d * font->cols * percent , 100 * 2) +
+			letter_space ;
+	}
+	else if( font->is_var_col_width)
+	{
+		return  0 ;
+	}
+	else if( strcmp( xft_size_type , XFT_SIZE) == 0)
+	{
+		double  widthpix ;
+		double  widthmm ;
+		u_int  dpi ;
+
+		widthpix = DisplayWidth( font->display , DefaultScreen(font->display)) ;
+		widthmm = DisplayWidthMM( font->display , DefaultScreen(font->display)) ;
+
+		dpi = DIVIDE_ROUNDING(widthpix * 254 , widthmm * 10) ;
+
+		return  DIVIDE_ROUNDINGUP(dpi * fontsize_d * font->cols , 72 * 2) + letter_space ;
+	}
+	else
+	{
+		return  DIVIDE_ROUNDINGUP(fontsize_d * font->cols , 2) + letter_space ;
+	}
 }
 
 static int
@@ -500,26 +531,18 @@ set_xft_font(
 	const char *  fontname ,
 	u_int  fontsize ,
 	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set. */
-	int  use_medium_for_bold ,
-	int  is_aa
+	int  use_medium_for_bold ,	/* Not used for now. */
+	u_int  letter_space ,
+	int  aa_opt		/* 0 = default , 1 = enable , -1 = disable */
 	)
 {
 	int  weight ;
 	int  slant ;
-	cs_info_t *  csinfo ;
 	u_int  ch_width ;
 	XftFont *  xfont ;
-	char **  font_encoding_p ;
-
-	if( ( csinfo = get_cs_info( FONT_CS(font->id))) == NULL)
-	{
-	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " charset(0x%.2x) is not supported.\n" ,
-			FONT_CS(font->id)) ;
-	#endif
-
-		return  0 ;
-	}
+	char *  iso10646 ;
+	
+	iso10646 = "iso10646-1" ;
 
 	/*
 	 * weight and slant can be modified in parse_xft_font_name().
@@ -540,8 +563,8 @@ set_xft_font(
 	{
 		char *  p ;
 		char *  font_family ;
-		char *  font_encoding ;
 		double  fontsize_d ;
+		char *  font_encoding ;
 		u_int  percent ;
 
 		if( ( p = kik_str_alloca_dup( fontname)) == NULL)
@@ -553,8 +576,8 @@ set_xft_font(
 			return  0 ;
 		}
 
-		font_encoding = "iso10646-1" ;
 		fontsize_d = (double)fontsize ;
+		font_encoding = iso10646 ;
 		percent = 0 ;
 		if( parse_xft_font_name( &font_family , &weight , &slant , &fontsize_d ,
 						&font_encoding , &percent , p))
@@ -563,15 +586,8 @@ set_xft_font(
 			{
 				/* basic font (e.g. usascii) width */
 
-				if( percent == 0)
-				{
-					ch_width = get_xft_col_width( font , font_family ,
-							weight , slant , fontsize_d) ;
-				}
-				else
-				{
-					ch_width = (fontsize_d * font->cols * percent) / 200 ;
-				}
+				ch_width = get_xft_col_width( font , fontsize_d , percent ,
+								letter_space) ;
 
 				if( font->is_vertical)
 				{
@@ -598,41 +614,49 @@ set_xft_font(
 				}
 			}
 
-		#if  0
-			kik_debug_printf( "Loading font %s%s%s %f\n" , font_family ,
+		#ifdef  DEBUG
+			kik_debug_printf( "Loading font %s%s%s %f %d\n" , font_family ,
 				weight == XFT_WEIGHT_BOLD ? ":Bold" :
 					weight == XFT_WEIGHT_LIGHT ? " Light" : "" ,
 				slant == XFT_SLANT_ITALIC ? ":Italic" : "" ,
-				fontsize_d) ;
+				fontsize_d , font->is_var_col_width) ;
 		#endif
 		
-			if( font->is_var_col_width)
+			if( ch_width == 0)
 			{
-				if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+				/* Proportional (font->is_var_col_width is true) */
+
+				if( ( xfont = XftFontOpen( font->display ,
+						DefaultScreen( font->display) ,
 						XFT_FAMILY , XftTypeString , font_family ,
-						XFT_PIXEL_SIZE , XftTypeDouble , fontsize_d ,
+						xft_size_type , XftTypeDouble , fontsize_d ,
 						XFT_ENCODING , XftTypeString , font_encoding ,
 						XFT_WEIGHT , XftTypeInteger , weight ,
 						XFT_SLANT , XftTypeInteger , slant ,
 						XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL ,
-						XFT_ANTIALIAS , XftTypeBool , is_aa ? True : False ,
-						NULL)))
+						aa_opt ? XFT_ANTIALIAS : NULL , XftTypeBool ,
+							aa_opt == 1 ? True : False , NULL)))
 				{
+					/* letter_space is ignored in variable column width mode */
+					ch_width = xft_calculate_char_width( font->display ,
+							xfont , "W" , 1) ;
+
 					goto  font_found ;
 				}
 			}
 			else
 			{
-				if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+				if( ( xfont = XftFontOpen( font->display ,
+						DefaultScreen( font->display) ,
 						XFT_FAMILY , XftTypeString , font_family ,
-						XFT_PIXEL_SIZE , XftTypeDouble , fontsize_d ,
+						xft_size_type , XftTypeDouble , fontsize_d ,
 						XFT_ENCODING , XftTypeString , font_encoding ,
 						XFT_WEIGHT , XftTypeInteger , weight ,
 						XFT_SLANT , XftTypeInteger , slant ,
-						XFT_CHAR_WIDTH , XftTypeInteger , ch_width ,
 						XFT_SPACING , XftTypeInteger , XFT_MONO ,
-						XFT_ANTIALIAS , XftTypeBool , is_aa ? True : False ,
-						NULL)))
+						XFT_CHAR_WIDTH , XftTypeInteger , ch_width ,
+						aa_opt ? XFT_ANTIALIAS : NULL , XftTypeBool ,
+							aa_opt == 1 ? True : False , NULL)) )
 				{
 					goto  font_found ;
 				}
@@ -646,14 +670,15 @@ set_xft_font(
 	if( col_width == 0)
 	{
 		/* basic font (e.g. usascii) width */
-		ch_width = get_xft_col_width( font , "" , weight , slant , (double)fontsize) ;
 
-		/*
-		 * !! Notice !!
-		 * The width of full and half character font is the same.
-		 */
+		ch_width = get_xft_col_width( font , (double)fontsize , 0 , letter_space) ;
+	
 		if( font->is_vertical)
 		{
+			/*
+			 * !! Notice !!
+			 * The width of full and half character font is the same.
+			 */
 			ch_width *= 2 ;
 		}
 	}
@@ -673,36 +698,39 @@ set_xft_font(
 		}
 	}
 
-	FOREACH_FONT_ENCODINGS(csinfo,font_encoding_p)
+	if( ch_width == 0)
 	{
-		if( font->is_var_col_width)
+		/* Proportional (font->is_var_col_width is true) */
+
+		if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+				xft_size_type , XftTypeDouble , (double)fontsize ,
+				XFT_ENCODING , XftTypeString , iso10646 ,
+				XFT_WEIGHT , XftTypeInteger , weight ,
+				XFT_SLANT , XftTypeInteger , slant ,
+				XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL ,
+				aa_opt ? XFT_ANTIALIAS : NULL , XftTypeBool ,
+					aa_opt == 1 ? True : False , NULL)))
 		{
-			if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
-					XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
-					XFT_ENCODING , XftTypeString , *font_encoding_p ,
-					XFT_WEIGHT , XftTypeInteger , weight ,
-					XFT_SLANT , XftTypeInteger , slant ,
-					XFT_SPACING , XftTypeInteger , XFT_PROPORTIONAL ,
-					XFT_ANTIALIAS , XftTypeBool , is_aa ? True : False ,
-					NULL)))
-			{
-				goto  font_found ;
-			}
+			/* letter_space is ignored in variable column width mode. */
+			ch_width = xft_calculate_char_width( font->display ,
+					xfont , "W" , 1) ;
+
+			goto  font_found ;
 		}
-		else
+	}
+	else
+	{
+		if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
+				xft_size_type , XftTypeDouble , (double)fontsize ,
+				XFT_ENCODING , XftTypeString , iso10646 ,
+				XFT_WEIGHT , XftTypeInteger , weight ,
+				XFT_SLANT , XftTypeInteger , slant ,
+				XFT_SPACING , XftTypeInteger , XFT_MONO ,
+				XFT_CHAR_WIDTH , XftTypeInteger , ch_width ,
+				aa_opt ? XFT_ANTIALIAS : NULL , XftTypeBool ,
+					aa_opt == 1 ? True : False , NULL)) )
 		{
-			if( ( xfont = XftFontOpen( font->display , DefaultScreen( font->display) ,
-					XFT_PIXEL_SIZE , XftTypeDouble , (double)fontsize ,
-					XFT_ENCODING , XftTypeString , *font_encoding_p ,
-					XFT_WEIGHT , XftTypeInteger , weight ,
-					XFT_SLANT , XftTypeInteger , slant ,
-					XFT_CHAR_WIDTH , XftTypeInteger , ch_width ,
-					XFT_SPACING , XftTypeInteger , XFT_CHARCELL ,
-					XFT_ANTIALIAS , XftTypeBool , is_aa ? True : False ,
-					NULL)))
-			{
-				goto  font_found ;
-			}
+			goto  font_found ;
 		}
 	}
 
@@ -930,7 +958,8 @@ set_xfont(
 	const char *  fontname ,
 	u_int  fontsize ,
 	u_int  col_width ,	/* if usascii font wants to be set , 0 will be set */
-	int  use_medium_for_bold
+	int  use_medium_for_bold ,
+	u_int  letter_space
 	)
 {
 	XFontStruct *  xfont ;
@@ -1201,11 +1230,11 @@ font_found:
 				 * !! Notice !!
 				 * The width of full and half character font is the same.
 				 */
-				ch_width = fontsize * percent / 100 ;
+				ch_width = DIVIDE_ROUNDING( fontsize * percent , 100) ;
 			}
 			else
 			{
-				ch_width = fontsize * percent / 200 ;
+				ch_width = DIVIDE_ROUNDING( fontsize * percent , 200) ;
 			}
 
 			if( font->width != ch_width)
@@ -1214,6 +1243,19 @@ font_found:
 
 				if( font->width < ch_width)
 				{
+					/*
+					 * If width(2) of '1' doesn't match ch_width(4)
+					 * x_off = (4-2)/2 = 1.
+					 * It means that starting position of drawing '1' is 1
+					 * as follows.
+					 *
+					 *  0123
+					 * +----+
+					 * | ** |
+					 * |  * |
+					 * |  * |
+					 * +----+
+					 */
 					font->x_off = (ch_width - font->width) / 2 ;
 				}
 
@@ -1231,6 +1273,14 @@ font_found:
 			font->x_off = font->width / 2 ;
 			font->width *= 2 ;
 		}
+
+		/* letter_space is ignored in variable column width mode. */
+		if( ! font->is_var_col_width && letter_space > 0)
+		{
+			font->is_proportional = 1 ;
+			font->width += letter_space ;
+			font->x_off += (letter_space / 2) ;
+		}
 	}
 	else
 	{
@@ -1296,7 +1346,7 @@ font_found:
 		font->is_proportional = 1 ;
 
 		/* XXX this may be inaccurate. */
-		font->width = (fontsize / 2) * font->cols ;
+		font->width = DIVIDE_ROUNDINGUP( fontsize * font->cols , 2) ;
 	}
 
 	if( font->height == 0)
@@ -1336,7 +1386,6 @@ x_compose_dec_special_font(void)
 	return  1 ;
 }
 
-
 x_font_t *
 x_font_new(
 	Display *  display ,
@@ -1346,7 +1395,8 @@ x_font_new(
 	const char *  fontname ,
 	u_int  fontsize ,
 	u_int  col_width ,
-	int  use_medium_for_bold
+	int  use_medium_for_bold ,
+	u_int  letter_space		/* ignored in variable column width mode. */
 	)
 {
 	x_font_t *  font ;
@@ -1406,7 +1456,9 @@ x_font_new(
 #ifdef  USE_TYPE_XFT
 	case  TYPE_XFT:
 		if( ! set_xft_font( font , fontname , fontsize , col_width , use_medium_for_bold ,
-			(font_present & FONT_AA) == FONT_AA))
+				letter_space ,
+				(font_present & FONT_AA) == FONT_AA ?
+					1 : ((font_present & FONT_NOAA) == FONT_NOAA ? -1 : 0)))
 		{
 			free( font) ;
 
@@ -1422,7 +1474,8 @@ x_font_new(
 		{
 			return  NULL ;
 		}
-		else if( ! set_xfont( font , fontname , fontsize , col_width , use_medium_for_bold))
+		else if( ! set_xfont( font , fontname , fontsize , col_width ,
+				use_medium_for_bold , letter_space))
 		{
 			free( font) ;
 
@@ -1433,6 +1486,10 @@ x_font_new(
 #endif
 	}
 
+#ifdef  DEBUG
+	x_font_dump( font) ;
+#endif
+
 	return  font ;
 }
 
@@ -1500,7 +1557,7 @@ x_calculate_char_width(
 	mkf_charset_t  cs
 	)
 {
-	if( font->is_var_col_width && ! font->decsp_font)
+	if( font->is_var_col_width && font->is_proportional && ! font->decsp_font)
 	{
 	#ifdef  USE_TYPE_XFT
 		if( font->xft_font)
@@ -1518,7 +1575,8 @@ x_calculate_char_width(
 				len = 4 ;
 			}
 
-			return  xft_calculate_char_width( font->display , font->xft_font , ch , len) ;
+			return  xft_calculate_char_width( font->display , font->xft_font ,
+								ch , len) ;
 		}
 	#endif
 	#ifdef  USE_TYPE_XCORE
@@ -1551,6 +1609,24 @@ x_font_get_encoding_names(
 	}
 }
 
+/* For mlterm-libvte */
+void
+x_font_use_point_size_for_xft(
+	int  bool
+	)
+{
+#ifdef  USE_TYPE_XFT
+	if( bool)
+	{
+		xft_size_type = XFT_SIZE ;
+	}
+	else
+	{
+		xft_size_type = XFT_PIXEL_SIZE ;
+	}
+#endif
+}
+
 #ifdef  DEBUG
 
 int
@@ -1559,16 +1635,35 @@ x_font_dump(
 	)
 {
 #ifdef  USE_TYPE_XCORE
-	kik_msg_printf( "  id %x: XFont %p" , font->id , font->xfont) ;
+	kik_msg_printf( "Font id %x: XFont %p" , font->id , font->xfont) ;
 #endif
 #ifdef  USE_TYPE_XFT
-	kik_msg_printf( " XftFont %p" , font->id , font->xft_font) ;
+	kik_msg_printf( "Font id %x: XftFont %p" , font->id , font->xft_font) ;
 #endif
 
+	kik_msg_printf( " (width %d, height %d, height_to_baseline %d, x_off %d)" ,
+		font->width , font->height , font->height_to_baseline , font->x_off) ;
+
 	if( font->is_proportional)
 	{
 		kik_msg_printf( " (proportional)") ;
 	}
+
+	if( font->is_var_col_width)
+	{
+		kik_msg_printf( " (var col width)") ;
+	}
+
+	if( font->is_vertical)
+	{
+		kik_msg_printf( " (vertical)") ;
+	}
+	
+	if( font->is_double_drawing)
+	{
+		kik_msg_printf( " (double drawing)") ;
+	}
+
 	kik_msg_printf( "\n") ;
 
 	return  1 ;
