@@ -85,6 +85,17 @@ init_usascii_font(
 	return  1 ;
 }
 
+static KIK_MAP( x_font)
+xfont_table_new(void)
+{
+	KIK_MAP( x_font)  xfont_table ;
+	
+	kik_map_new_with_size( ml_font_t , x_font_t * , xfont_table ,
+		font_hash , font_compare , 16) ;
+
+	return  xfont_table ;
+}
+
 static int
 xfont_table_delete(
 	KIK_MAP( x_font)  xfont_table
@@ -118,7 +129,8 @@ x_acquire_font_cache(
 	u_int  font_size ,
 	mkf_charset_t  usascii_font_cs ,
 	x_font_config_t *  font_config ,
-	int  use_multi_col_char
+	int  use_multi_col_char ,
+	u_int  letter_space
 	)
 {
 	int  count ;
@@ -131,7 +143,8 @@ x_acquire_font_cache(
 			font_caches[count]->font_size == font_size &&
 			font_caches[count]->usascii_font_cs == usascii_font_cs &&
 			font_caches[count]->font_config == font_config &&
-			font_caches[count]->use_multi_col_char == use_multi_col_char)
+			font_caches[count]->use_multi_col_char == use_multi_col_char &&
+			font_caches[count]->letter_space == letter_space)
 		{
 			font_caches[count]->ref_count ++ ;
 
@@ -153,13 +166,13 @@ x_acquire_font_cache(
 
 	font_cache->font_config = font_config ;
 
-	kik_map_new_with_size( ml_font_t , x_font_t * ,
-		font_cache->xfont_table , font_hash , font_compare , 32) ;
+	font_cache->xfont_table = xfont_table_new() ;
 
 	font_cache->display = display ;
 	font_cache->font_size = font_size ;
 	font_cache->usascii_font_cs = usascii_font_cs ;
 	font_cache->use_multi_col_char = use_multi_col_char ;
+	font_cache->letter_space = letter_space ;
 	font_cache->ref_count = 1 ;
 
 	if( ! init_usascii_font( font_cache))
@@ -226,8 +239,8 @@ x_font_cache_unload(
 	/*
 	 * Creating new cache.
 	 */
-	kik_map_new_with_size( ml_font_t , x_font_t * ,
-		font_cache->xfont_table , font_hash , font_compare , 32) ;
+	font_cache->xfont_table = xfont_table_new() ;
+
 	if( ! init_usascii_font( font_cache))
 	{
 	#ifdef  DEBUG
@@ -308,9 +321,11 @@ x_font_cache_get_xfont(
 		}
 	}
 
-	if( ( xfont = x_font_new( font_cache->display , font , font_cache->font_config->type_engine ,
-			font_cache->font_config->font_present , fontname ,
-			font_cache->font_size , col_width , use_medium_for_bold)))
+	if( ( xfont = x_font_new( font_cache->display , font ,
+				font_cache->font_config->type_engine ,
+				font_cache->font_config->font_present , fontname ,
+				font_cache->font_size , col_width , use_medium_for_bold ,
+				font_cache->letter_space)))
 	{
 		if( ! font_cache->use_multi_col_char)
 		{
