@@ -864,6 +864,9 @@ window_realized(
 
 	cand_screen = (x_im_candidate_screen_t*) win ;
 
+	x_window_set_xft( &cand_screen->window ,
+		x_get_type_engine( cand_screen->font_man) == TYPE_XFT) ;
+
 	x_window_set_fg_color( win , x_get_xcolor( cand_screen->color_man , ML_FG_COLOR)) ;
 	x_window_set_bg_color( win , x_get_xcolor( cand_screen->color_man , ML_BG_COLOR)) ;
 
@@ -1053,10 +1056,10 @@ x_im_candidate_screen_new(
 	cand_screen->listener.selected = NULL ;
 
 	if( ! x_display_show_root( disp , &cand_screen->window , x , y , 0 ,
-					  "mlterm-candidate-window"))
+					  "mlterm-candidate-window" , 0))
 	{
 	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " x_window_manager_show_root() failed.\n") ;
+		kik_warn_printf( KIK_DEBUG_TAG " x_display_show_root() failed.\n") ;
 	#endif
 
 		goto  error ;
