@@ -254,6 +254,9 @@ window_realized(
 
 	stat_screen = (x_im_status_screen_t*) win ;
 
+	x_window_set_xft( &stat_screen->window ,
+		x_get_type_engine( stat_screen->font_man) == TYPE_XFT) ;
+
 	x_window_set_fg_color( win , x_get_xcolor( stat_screen->color_man , ML_FG_COLOR)) ;
 	x_window_set_bg_color( win , x_get_xcolor( stat_screen->color_man , ML_BG_COLOR)) ;
 
@@ -368,10 +371,8 @@ x_im_status_screen_new(
 	stat_screen->set_spot = set_spot ;
 	stat_screen->set = set ;
 
-	if( ! x_display_show_root( disp ,
-					&stat_screen->window ,
-					x , y , 0 ,
-					"mlterm-status-window"))
+	if( ! x_display_show_root( disp , &stat_screen->window ,
+					x , y , 0 , "mlterm-status-window" , 0))
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_display_show_root() failed.\n") ;
