@@ -9,6 +9,12 @@
 #include  <kiklib/kik_str.h>	/* kik_str_to_uint */
 
 
+/* --- static variables --- */
+
+static char *  true = "true" ;
+static char *  false = "false" ;
+
+
 /* --- global functions --- */
 
 int
@@ -17,7 +23,7 @@ x_prepare_for_main_config(
 	)
 {
 	char *  rcpath ;
-	
+
 	/*
 	 * XXX
 	 * "mlterm/core" is for backward compatibility with 1.9.44
@@ -34,7 +40,7 @@ x_prepare_for_main_config(
 		kik_conf_read( conf , rcpath) ;
 		free( rcpath) ;
 	}
-	
+
 	if( ( rcpath = kik_get_sys_rc_path( "mlterm/main")))
 	{
 		kik_conf_read( conf , rcpath) ;
@@ -70,12 +76,10 @@ x_prepare_for_main_config(
 		"screen width in percent against font width [100]") ;
 	kik_conf_add_opt( conf , '2' , "hscr" , 0 , "screen_height_ratio" ,
 		"screen height in percent against font height [100]") ;
-#if defined(USE_IMLIB) || defined(USE_GDK_PIXBUF)
 	kik_conf_add_opt( conf , '3' , "contrast" , 0 , "contrast" ,
 		"contrast of background image in percent [100]") ;
 	kik_conf_add_opt( conf , '4' , "gamma" , 0 , "gamma" ,
 		"gamma of background image in percent [100]") ;
-#endif
 	kik_conf_add_opt( conf , '5' , "big5bug" , 1 , "big5_buggy" ,
 		"manage buggy Big5 CTEXT in XFree86 4.1 or earlier [false]") ;
 	kik_conf_add_opt( conf , '6' , "stbs" , 1 , "static_backscroll_mode" ,
@@ -90,7 +94,7 @@ x_prepare_for_main_config(
 		"cursor background color") ;
 #ifdef  USE_TYPE_XFT
 	kik_conf_add_opt( conf , 'A' , "aa" , 1 , "use_anti_alias" , 
-		"use anti-alias font by using Xft [false]") ;
+		"forcibly use anti alias font by using Xft") ;
 #endif
 	kik_conf_add_opt( conf , 'B' , "sbbg" , 0 , "sb_bg_color" , 
 		"scrollbar background color") ;
@@ -108,10 +112,8 @@ x_prepare_for_main_config(
 		"scrollbar foreground color") ;
 	kik_conf_add_opt( conf , 'G' , "vertical" , 0 , "vertical_mode" ,
 		"vertical mode (none/cjk/mongol) [none]") ;
-#if defined(USE_IMLIB) || defined(USE_GDK_PIXBUF)
 	kik_conf_add_opt( conf , 'H' , "bright" , 0 , "brightness" ,
 		"brightness of background image in percent [100]") ;
-#endif
 #ifndef  USE_WIN32GUI
 	kik_conf_add_opt( conf , 'I' , "icon" , 0 , "icon_name" , 
 		"icon name") ;
@@ -164,7 +166,7 @@ x_prepare_for_main_config(
 		"use non-Unicode fonts even in UTF-8 mode [false]") ;
 	kik_conf_add_opt( conf , 'o' , "lsp" , 0 , "line_space" ,
 		"extra space between lines in pixels [0]") ;
-#if defined(USE_IMLIB) || defined(USE_GDK_PIXBUF)
+#ifdef  USE_EXT_IMAGELIB
 	kik_conf_add_opt( conf , 'p' , "pic" , 0 , "wall_picture" , 
 		"path for wallpaper (background) image") ;
 #endif
@@ -186,18 +188,26 @@ x_prepare_for_main_config(
 		"terminal type for TERM variable [xterm]") ;
 	kik_conf_add_opt( conf , 'z' ,  "largesmall" , 0 , "step_in_changing_font_size" ,
 		"step in changing font size in GUI configurator [1]") ;
-#if defined(USE_IMLIB) || defined(USE_GDK_PIXBUF)
+#ifdef  USE_EXT_IMAGELIB
 	kik_conf_add_opt( conf , '\0' , "iconpath" , 0 , "icon_path" ,
 		"path to an imagefile to be use as an window icon") ;
+#endif
+#ifdef  USE_FRIBIDI
+	kik_conf_add_opt( conf , '\0' , "bimode" , 0 , "bidi_mode" ,
+		"bidi mode [normal]") ;
 #endif
 	kik_conf_add_opt( conf , '\0' , "im" , 0 , "input_method" ,
 		"input method (xim/kbd/uim/iiimf/m17nlib/scim/none) [xim]") ;
+	kik_conf_add_opt( conf , '\0' , "parent" , 0 , "parent_window" ,
+		"parent window") ;
 #ifdef  USE_WIN32API
 	kik_conf_add_opt( conf , '\0' , "servlist" , 0 , "server_list" ,
 		"list of servers to connect") ;
 	kik_conf_add_opt( conf , '\0' , "serv" , 0 , "default_server" ,
 		"connecting server by default") ;
 #endif
+	kik_conf_add_opt( conf , '\0' , "csp" , 0 , "letter_space" ,
+		"extra space between letters in pixels [0]") ;
 	kik_conf_set_end_opt( conf , 'e' , NULL , "exec_cmd" , 
 		"execute external command") ;
 
@@ -308,7 +318,7 @@ x_main_config_init(
 	
 	if( ( value = kik_conf_get_value( conf , "use_combining")))
 	{
-		if( strcmp( value , "false") == 0)
+		if( strcmp( value , false) == 0)
 		{
 			main_config->use_char_combining = 0 ;
 		}
@@ -317,7 +327,7 @@ x_main_config_init(
 	main_config->use_dynamic_comb = 0 ;
 	if( ( value = kik_conf_get_value( conf , "use_dynamic_comb")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->use_dynamic_comb = 1 ;
 		}
@@ -326,7 +336,7 @@ x_main_config_init(
 	main_config->logging_vt_seq = 0 ;
 	if( ( value = kik_conf_get_value( conf , "logging_vt_seq")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->logging_vt_seq = 1 ;
 		}
@@ -336,7 +346,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_variable_column_width")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->font_present |= FONT_VAR_WIDTH ;
 		}
@@ -389,12 +399,16 @@ x_main_config_init(
 #ifdef  USE_TYPE_XFT
 	if( ( value = kik_conf_get_value( conf , "use_anti_alias")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->font_present |= FONT_AA ;
 			/* forcibly use xft */
 			main_config->type_engine = TYPE_XFT ;
 		}
+		else if( strcmp( value , false) == 0)
+		{
+			main_config->font_present |= FONT_NOAA ;
+		}
 	}
 #endif
 
@@ -551,7 +565,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_multi_column_char")))
 	{
-		if( strcmp( value , "false") == 0)
+		if( strcmp( value , false) == 0)
 		{
 			main_config->use_multi_col_char = 0 ;
 		}
@@ -573,6 +587,22 @@ x_main_config_init(
 		}
 	}
 
+	main_config->letter_space = 0 ;
+	
+	if( ( value = kik_conf_get_value( conf , "letter_space")))
+	{
+		u_int  size ;
+		
+		if( kik_str_to_uint( &size , value))
+		{
+			main_config->letter_space = size ;
+		}
+		else
+		{
+			kik_msg_printf( invalid_msg , "letter space" , value) ;
+		}
+	}
+
 	if( ( value = kik_conf_get_value( conf , "logsize")) == NULL)
 	{
 		main_config->num_of_log_lines = 128 ;
@@ -602,7 +632,7 @@ x_main_config_init(
 	
 	if( ( value = kik_conf_get_value( conf , "use_login_shell")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->use_login_shell = 1 ;
 		}
@@ -612,7 +642,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "big5_buggy")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->big5_buggy = 1 ;
 		}
@@ -622,7 +652,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_scrollbar")))
 	{
-		if( strcmp( value , "false") == 0)
+		if( strcmp( value , false) == 0)
 		{
 			main_config->use_scrollbar = 0 ;
 		}
@@ -641,7 +671,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "iso88591_font_for_usascii")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->iso88591_font_for_usascii = 1 ;
 		}
@@ -651,7 +681,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "not_use_unicode_font")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->unicode_font_policy = NOT_USE_UNICODE_FONT ;
 		}
@@ -659,7 +689,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "only_use_unicode_font")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			if( main_config->unicode_font_policy == NOT_USE_UNICODE_FONT)
 			{
@@ -681,7 +711,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "receive_string_via_ucs")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->receive_string_via_ucs = 1 ;
 		}
@@ -718,7 +748,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_transbg")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->use_transbg = 1 ;
 		}
@@ -841,12 +871,19 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_bidi")))
 	{
-		if( strcmp( value , "false") == 0)
+		if( strcmp( value , false) == 0)
 		{
 			main_config->use_bidi = 0 ;
 		}
 	}
 
+	main_config->bidi_mode = BIDI_NORMAL_MODE ;
+	
+	if( ( value = kik_conf_get_value( conf , "bidi_mode")))
+	{
+		main_config->bidi_mode = ml_get_bidi_mode( value) ;
+	}
+
 	/* If value is "none" or not is also checked in x_screen.c */
 	if( ( value = kik_conf_get_value( conf , "mod_meta_key")) &&
 		strcmp( value , "none") != 0)
@@ -880,7 +917,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "use_vertical_cursor")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->use_vertical_cursor = 1 ;
 		}
@@ -902,7 +939,7 @@ x_main_config_init(
 	
 	if( ( value = kik_conf_get_value( conf , "use_extended_scroll_shortcut")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->use_extended_scroll_shortcut = 1 ;
 		}
@@ -912,7 +949,7 @@ x_main_config_init(
 	
 	if( ( value = kik_conf_get_value( conf , "borderless")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->borderless = 1 ;
 		}
@@ -922,7 +959,7 @@ x_main_config_init(
 
 	if( ( value = kik_conf_get_value( conf , "static_backscroll_mode")))
 	{
-		if( strcmp( value , "true") == 0)
+		if( strcmp( value , true) == 0)
 		{
 			main_config->bs_mode = BSM_STATIC ;
 		}
@@ -1003,6 +1040,22 @@ x_main_config_init(
 		main_config->init_str = NULL ;
 	}
 
+	main_config->parent_window = 0 ;
+	
+	if( ( value = kik_conf_get_value( conf , "parent_window")))
+	{
+		u_int  parent_window ;
+		
+		if( kik_str_to_uint( &parent_window , value))
+		{
+			main_config->parent_window = parent_window ;
+		}
+		else
+		{
+			kik_msg_printf( invalid_msg , "parent window" , value) ;
+		}
+	}
+	
 #ifdef  USE_WIN32API
 	main_config->server_list = NULL ;
 	
@@ -1046,7 +1099,7 @@ x_main_config_init(
 	}
 #endif
 
-	if( ( value = kik_conf_get_value( conf , "exec_cmd")) && strcmp( value , "true") == 0)
+	if( ( value = kik_conf_get_value( conf , "exec_cmd")) && strcmp( value , true) == 0)
 	{
 		if( ( main_config->cmd_argv = malloc( sizeof( char*) * (argc + 1))) == NULL)
 		{
