@@ -41,7 +41,7 @@ typedef struct x_system_event_listener
 	ml_term_t *  (*get_pty)( void * , char *) ;
 	char *  (*pty_list)( void *) ;
 
-	int  (*mlclient)( void * , x_screen_ptr_t , char * , size_t , FILE *) ;
+	int  (*mlclient)( void * , x_screen_ptr_t , char * , FILE *) ;
 
 	void  (*font_config_updated)(void) ;
 	void  (*color_config_updated)(void) ;
@@ -111,8 +111,6 @@ typedef struct  x_screen
 
 	x_bel_mode_t  bel_mode ;
 
-	u_int  line_space ;
-
 	u_int  screen_width_ratio ;
 	u_int  screen_height_ratio ;
 
@@ -132,6 +130,7 @@ typedef struct  x_screen
 
 	char *  pic_file_path ;
 	x_picture_modifier_t  pic_mod ;
+	x_bg_picture_t *  bg_pic ;
 
 	x_icon_picture_t *  icon ;
 
@@ -144,6 +143,7 @@ typedef struct  x_screen
 	char *  conf_menu_path_3 ;
 
 	u_int8_t  fade_ratio ;
+	u_int8_t  line_space ;
 	int8_t  receive_string_via_ucs ;
 	int8_t  use_vertical_cursor ;
 	int8_t  use_extended_scroll_shortcut ;
@@ -154,7 +154,7 @@ typedef struct  x_screen
 
 
 /* xterm = traditional, menuN = conf_menu_path_N */
-int  x_set_button3_behavior( char *  mode) ;
+int  x_set_button3_behavior( const char *  mode) ;
 
 x_screen_t *  x_screen_new( ml_term_t *  term , x_font_manager_t *  font_man ,
 	x_color_manager_t *  color_man , x_termcap_entry_t *  termcap ,
@@ -172,6 +172,8 @@ int  x_screen_delete( x_screen_t *  screen) ;
 
 int  x_screen_attach( x_screen_t *  screen , ml_term_t *  term) ;
 
+int  x_screen_attached( x_screen_t *  screen) ;
+
 ml_term_t *  x_screen_detach( x_screen_t *  screen) ;
 
 int  x_set_system_listener( x_screen_t *  screen ,
@@ -201,8 +203,6 @@ u_int  x_line_bottom_margin( x_screen_t *  screen) ;
 
 void  x_screen_set_config( x_screen_t *  screen, char *  dev , char *  key , char *  value) ;
 
-void  x_screen_get_config( x_screen_t *  screen , char *  dev , char *  key , int  to_menu) ;
-
 
 int  x_screen_reset_view( x_screen_t *  screen) ;
 
