@@ -1,5 +1,5 @@
 /*
- *	$Id: x_selection.h,v 1.3 2004/02/22 07:49:48 arakiken Exp $
+ *	$Id$
  */
 
 #ifndef  __X_SELECTION_H__
@@ -38,9 +38,8 @@ typedef struct  x_selection
 	int  prev_col ;
 	int  prev_row ;
 	
-	int  is_selecting ;
-	int  is_reversed ;
-	int  is_owner ;
+	int8_t  is_selecting ;
+	int8_t  is_reversed ;
 
 }  x_selection_t ;
 
