@@ -29,6 +29,7 @@
 #include  <kiklib/kik_net.h>	/* socket/bind/listen/sockaddr_un */
 #include  <kiklib/kik_types.h>	/* u_int */
 #include  <kiklib/kik_sig_child.h>
+#include  <kiklib/kik_args.h>	/* kik_arg_str_to_array */
 #include  <ml_term_manager.h>
 #include  <ml_char_encoding.h>
 
@@ -65,6 +66,8 @@ static u_int32_t *  dead_mask ;
 static x_screen_t **  screens ;
 static u_int  num_of_screens ;
 
+static u_int  depth ;
+
 static u_int  num_of_startup_screens ;
 static u_int  num_of_startup_ptys ;
 
@@ -242,9 +245,9 @@ create_term_intern(void)
 	if( ( term = ml_create_term( main_config.cols , main_config.rows ,
 			main_config.tab_size , main_config.num_of_log_lines ,
 			main_config.encoding , main_config.is_auto_encoding , 
-			main_config.unicode_font_policy ,
-			main_config.col_size_of_width_a , main_config.use_char_combining ,
-			main_config.use_multi_col_char , main_config.use_bidi ,
+			main_config.unicode_font_policy , main_config.col_size_of_width_a ,
+			main_config.use_char_combining , main_config.use_multi_col_char ,
+			main_config.use_bidi , main_config.bidi_mode ,
 			x_termcap_get_bool_field(
 				x_termcap_get_entry( &termcap , main_config.term_type) , ML_BCE) ,
 			main_config.use_dynamic_comb , main_config.bs_mode ,
@@ -602,7 +605,7 @@ open_screen_intern(
 		}
 	}
 
-	if( ( disp = x_display_open( main_config.disp_name)) == NULL)
+	if( ( disp = x_display_open( main_config.disp_name , depth)) == NULL)
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_display_open failed.\n") ;
@@ -632,12 +635,13 @@ open_screen_intern(
 		main_config.type_engine , main_config.font_present ,
 		main_config.font_size , usascii_font_cs ,
 		usascii_font_cs_changable , main_config.use_multi_col_char ,
-		main_config.step_in_changing_font_size)) == NULL)
+		main_config.step_in_changing_font_size ,
+		main_config.letter_space)) == NULL)
 	{
 		char **  names ;
 
 		names = x_font_get_encoding_names( usascii_font_cs) ;
-		if( names[0] == NULL)
+		if( names == NULL || names[0] == NULL)
 		{
 			kik_msg_printf(
 			  "Current encoding \"%s\" is supported only with "
@@ -645,7 +649,7 @@ open_screen_intern(
 			  "Please use \"-u\" option.\n",
 			  ml_get_char_encoding_name(ml_term_get_encoding(term))) ;
 		}
-		else if( strcmp( names[0] , "iso10646-1" ) == 0 )
+		else if( strcmp( names[0] , "iso10646-1") == 0)
 		{
 			kik_msg_printf(
 			  "No fonts found for charset \"%s\".  "
@@ -653,6 +657,10 @@ open_screen_intern(
 		}
 		else if( names[1] == NULL)
 		{
+			/*
+			 * names[0] exists but x_font_manager_new failed.
+			 * It means that no fonts for names[0] exit in your system.
+			 */
 			kik_msg_printf(
 			  "No fonts found for charset \"%s\".  "
 			  "Please install fonts or use Unicode font "
@@ -665,6 +673,7 @@ open_screen_intern(
 			  "Please install fonts or use Unicode font "
 			  "(\"-u\" option).\n" , names[0] , names[1] );
 		}
+
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_font_manager_new() failed.\n") ;
 	#endif
@@ -672,8 +681,7 @@ open_screen_intern(
 		goto  error ;
 	}
 
-	if( ( color_man = x_color_manager_new( disp->display ,
-				DefaultScreen( disp->display) , &color_config ,
+	if( ( color_man = x_color_manager_new( disp , &color_config ,
 				main_config.fg_color , main_config.bg_color ,
 				main_config.cursor_fg_color , main_config.cursor_bg_color)) == NULL)
 	{
@@ -732,7 +740,7 @@ open_screen_intern(
 	}
 
 	if( ! x_display_show_root( disp, root, main_config.x, main_config.y,
-		main_config.geom_hint, main_config.app_name))
+		main_config.geom_hint, main_config.app_name , main_config.parent_window))
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG " x_display_show_root() failed.\n") ;
@@ -1010,7 +1018,7 @@ prev_pty(
 		}
 	}
 }
-	
+
 static void
 pty_closed(
 	void *  p ,
@@ -1137,99 +1145,13 @@ mlclient(
 	void *  self ,
 	x_screen_t *  screen ,
 	char *  args ,
-	size_t  args_len ,	/* Including '\0' */
 	FILE *  fp		/* Stream to output response of mlclient. */
 	)
 {
 	char **  argv ;
 	int  argc ;
-	char *  args_dup ;
-	char *  p ;
-	
-	/*
-	 * parsing options.
-	 */
 
-	argc = 0 ;
-
-	if( ( argv = alloca( sizeof( char*) * ( args_len + 1))) == NULL)
-	{
-		return  0 ;
-	}
-
-	if( ( args_dup = alloca( args_len)) == NULL)
-	{
-		return  0 ;
-	}
-	
-	p = args_dup ;
-
-	while( *args)
-	{
-		int  quoted ;
-
-		while( *args == ' ' || *args == '\t')
-		{
-			if( *args == '\0')
-			{
-				goto  parse_end ;
-			}
-
-			args ++ ;
-		}
-
-		if( *args == '\"')
-		{
-			quoted = 1 ;
-			args ++ ;
-		}
-		else
-		{
-			quoted = 0 ;
-		}
-		
-		while( *args)
-		{
-			if( quoted)
-			{
-				if( *args == '\"')
-				{
-					args ++ ;
-					
-					break ;
-				}
-			}
-			else
-			{
-				if( *args == ' ' || *args == '\t')
-				{
-					args ++ ;
-					
-					break ;
-				}
-			}
-			
-			if( *args == '\\' && *(args + 1) == '\"')
-			{
-				*(p ++) = *(++ args) ;
-			}
-			else
-			{
-				*(p ++) = *args ;
-			}
-
-			args ++ ;
-		}
-
-		*(p ++) = '\0' ;
-		argv[argc ++] = args_dup ;
-		args_dup = p ;
-	}
-
-parse_end:
-
-	/* NULL terminator (POSIX exec family style) */
-	argv[argc] = NULL ;
+	argv = kik_arg_str_to_array( &argc , args) ;
 
 #ifdef  __DEBUG
 	{
@@ -1328,7 +1250,7 @@ parse_end:
 
 		if( screen)
 		{
-			open_pty( p , screen , pty) ;
+			open_pty( self , screen , pty) ;
 		}
 		else
 		{
@@ -1556,7 +1478,7 @@ client_connected(void)
 	kik_debug_printf( KIK_DEBUG_TAG " %s\n" , args) ;
 #endif
 
-	if( ! mlclient( NULL , NULL , args , line_len , fp))
+	if( ! mlclient( NULL , NULL , args , fp))
 	{
 		goto  error ;
 	}
@@ -1851,6 +1773,8 @@ x_term_manager_init(
 	#endif
 		) ;
 #endif
+	kik_conf_add_opt( conf , '\0' , "depth" , 0 , "depth" ,
+		"specify visual depth") ;
 	kik_conf_add_opt( conf , '\0' , "maxptys" , 0 , "max_ptys" ,
 		"max ptys to open simultaneously (multiple of 32)") ;
 	kik_conf_add_opt( conf , '\0' , "button3" , 0 , "button3_behavior" ,
@@ -1971,6 +1895,11 @@ x_term_manager_init(
 	}
 #endif
 
+	if( ( value = kik_conf_get_value( conf , "depth")))
+	{
+		kik_str_to_uint( &depth , value) ;
+	}
+	
 	max_screens_multiple = 1 ;
 
 	if( ( value = kik_conf_get_value( conf , "max_ptys")))
@@ -2062,7 +1991,7 @@ x_term_manager_init(
 	{
 		x_set_button3_behavior( value) ;
 	}
-	
+
 
 	if( ( version = kik_conf_get_version( conf)) == NULL)
 	{
@@ -2338,8 +2267,8 @@ x_term_manager_event_loop(void)
 
 int
 x_term_manager_add_fd(
-	int fd ,
-	void (*handler)(void)
+	int  fd ,
+	void  (*handler)(void)
 	)
 {
 	int  i ;
@@ -2365,7 +2294,9 @@ x_term_manager_add_fd(
 }
 
 int
-x_term_manager_remove_fd( int fd)
+x_term_manager_remove_fd(
+	int  fd
+	)
 {
 	int  i ;
 
