@@ -42,6 +42,7 @@
 #define  XA_TAKE_FOCUS(display) (XInternAtom(display , "WM_TAKE_FOCUS" , False))
 #define  XA_INCR(display) (XInternAtom(display, "INCR", False))
 #define  XA_XROOTPMAP_ID(display) (XInternAtom(display, "_XROOTPMAP_ID", False))
+#define  XA_XSETROOT_ID(display) (XInternAtom(display, "_XSETROOT_ID" , False))
 #define  XA_WM_CLIENT_LEADER(display) (XInternAtom(display , "WM_CLIENT_LEADER" , False))
 
 /*
@@ -56,6 +57,10 @@
 #define  XA_MWM_INFO(display) (XInternAtom(display, "_MOTIF_WM_INFO", True))
 #define  XA_MWM_HINTS(display) (XInternAtom(display, "_MOTIF_WM_HINTS", True))
 
+#define  IS_INHERIT_TRANSPARENT(win) \
+		( use_inherit_transparent && x_picture_modifier_is_normal( (win)->pic_mod))
+
+
 typedef struct {
 	u_int32_t  flags ;
 	u_int32_t  functions ;
@@ -70,8 +75,8 @@ typedef struct {
 #define  MAX_CLICK  3			/* max is triple click */
 
 
-#define  restore_fg_color(win)  x_gc_set_fg_color((win)->gc,(win)->fg_color)
-#define  restore_bg_color(win)  x_gc_set_bg_color((win)->gc,(win)->bg_color)
+#define  restore_fg_color(win)  x_gc_set_fg_color((win)->gc,(win)->fg_color.pixel)
+#define  restore_bg_color(win)  x_gc_set_bg_color((win)->gc,(win)->bg_color.pixel)
 
 
 #if  0
@@ -82,82 +87,15 @@ typedef struct {
 /* --- static variables --- */
 
 static int  click_interval = 250 ;	/* millisecond, same as xterm. */
+/* ParentRelative isn't used for transparency by default */
+static int  use_inherit_transparent = 0 ;
 
 
 /* --- static functions --- */
 
-/*
- * only for double buffering
- */
+/* Only used for set_transparent|update_modified_transparent */
 static int
-restore_view(
-	x_window_t *  win ,
-	int  x ,
-	int  y ,
-	u_int	width ,
-	u_int	height
-	)
-{
-	if( win->buffer == win->drawable)
-	{
-		if( win->width < x + width)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG
-				" update size (x)%d (w)%d is overflowed.(screen width %d)\n" ,
-				x , width , win->width) ;
-		#endif
-
-			width = win->width - x ;
-
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " width is modified -> %d\n" , width) ;
-		#endif
-		}
-
-		if( win->height < y + height)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG
-				" update size (y)%d (h)%d is overflowed.(screen height is %d)\n" ,
-				y , height , win->height) ;
-		#endif
-
-			height = win->height - y ;
-
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " height is modified -> %d\n" , height) ;
-		#endif
-		}
-
-		XCopyArea( win->disp->display , win->drawable , win->my_window , win->gc->gc ,
-			x + win->margin , y + win->margin , width , height ,
-			x + win->margin , y + win->margin) ;
-
-		return  1 ;
-	}
-	else
-	{
-		return  0 ;
-	}
-}
-
-#if  0
-/*
- * XXX
- * Not used for now.
- */
-static int
-restore_view_all(
-	x_window_t *  win
-	)
-{
-	return	restore_view( win , 0 , 0 , win->width , win->height) ;
-}
-#endif
-
-static int
-set_transparent(
+set_transparent_picture(
 	x_window_t *  win ,
 	Pixmap  pixmap
 	)
@@ -182,11 +120,49 @@ set_transparent(
 	return  1 ;
 }
 
+/* Only used for set_transparent */
+static int
+update_transparent_picture(
+	x_window_t *  win
+	)
+{
+	x_bg_picture_t *  pic ;
+
+	if( ! ( pic = x_acquire_bg_picture( win , win->pic_mod , "root")))
+	{
+		goto  error1 ;
+	}
+
+	if( ! set_transparent_picture( win , pic->pixmap))
+	{
+		goto  error2 ;
+	}
+
+	x_release_bg_picture( pic) ;
+
+	return  1 ;
+
+error2:
+	x_release_bg_picture( pic) ;
+
+error1:
+	win->is_transparent = 0 ;
+	win->pic_mod = NULL ;
+
+	return  0 ;
+}
+
 static int
 unset_transparent(
 	x_window_t *  win
 	)
 {
+	/*
+	 * XXX
+	 * If previous mode is not modified transparent,
+	 * ParentRelative mode of parent windows should be unset.
+	 */
+
 	/*
 	 * !! Notice !!
 	 * this must be done before x_window_unset_wall_picture() because
@@ -200,27 +176,89 @@ unset_transparent(
 }
 
 static int
-update_pic_transparent(
+set_transparent(
 	x_window_t *  win
 	)
 {
-	x_bg_picture_t  pic ;
+	Window  parent ;
 
-	if( ! x_bg_picture_init( &pic , win , win->pic_mod))
+	if( ! IS_INHERIT_TRANSPARENT(win))
 	{
-		return  0 ;
+		/*
+		 * XXX
+		 * If previous mode is not modified transparent,
+		 * ParentRelative mode of parent windows should be unset.
+		 */
+
+		if( x_root_pixmap_available( win->disp->display))
+		{
+			/*
+			 * win->is_transparent is set appropriately in
+			 * update_transparent_picture().
+			 */
+			return  update_transparent_picture( win) ;
+		}
+		else
+		{
+			kik_msg_printf( "_X_ROOTPMAP_ID is not found."
+				" Trying ParentRelative for transparency instead.\n") ;
+			
+			if( ! x_picture_modifier_is_normal( win->pic_mod))
+			{
+				kik_msg_printf( "(brightness, contrast, gamma and alpha "
+					"options are ignored)\n") ;
+				
+				win->pic_mod = NULL ;
+			}
+
+			use_inherit_transparent = 1 ;
+		}
 	}
 
-	if( ! x_bg_picture_get_transparency( &pic))
-	{
-		x_bg_picture_final( &pic) ;
+	/*
+	 * It is not necessary to set ParentRelative more than once, so
+	 * this function should be used as follows.
+	 * if( ! IS_INHERIT_TRANSPARENT(win) || ! win->wall_picture_is_set)
+	 * {
+	 *	set_transparent( win) ;
+	 * }
+	 */
+	
+	/*
+	 * Root - Window A - Window C
+	 *      - Window B - Window D
+	 *                 - Window E
+	 * If Window C is set_transparent(), C -> A -> Root are set ParentRelative.
+	 * Window B,D and E are not set ParentRelative.
+	 */
 
-		return  0 ;
+	while( win->parent)
+	{
+		/* win->is_transparent is set appropriately in set_transparent() */
+		set_transparent_picture( win , ParentRelative) ;
+
+		win = win->parent ;
 	}
 
-	set_transparent( win , pic.pixmap) ;
+	set_transparent_picture( win , ParentRelative) ;
 
-	x_bg_picture_final( &pic) ;
+	parent = win->my_window ;
+	while( 1)
+	{
+		Window  root ;
+		Window *  list ;
+		u_int  n ;
+
+		XQueryTree( win->disp->display , parent , &root , &parent , &list , &n) ;
+		XFree( list) ;
+
+		if( parent == DefaultRootWindow( win->disp->display))
+		{
+			break ;
+		}
+
+		XSetWindowBackgroundPixmap( win->disp->display , parent , ParentRelative) ;
+	}
 
 	return  1 ;
 }
@@ -230,7 +268,7 @@ notify_focus_in_to_children(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	if( ! win->is_focused)
 	{
@@ -255,7 +293,7 @@ notify_focus_out_to_children(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	if( win->is_focused)
 	{
@@ -280,13 +318,16 @@ notify_configure_to_children(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	if( win->is_transparent)
 	{
-		if( win->pic_mod || x_root_pixmap_available( win->disp->display))
+		if( ! IS_INHERIT_TRANSPARENT(win) || ! win->wall_picture_is_set)
 		{
-			update_pic_transparent( win) ;
+		#ifdef  __DEBUG
+			kik_debug_printf( "configure notify for transparency\n") ;
+		#endif
+			set_transparent( win) ;
 		}
 		else
 		{
@@ -308,13 +349,18 @@ notify_reparent_to_children(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	if( win->is_transparent)
 	{
-		x_window_set_transparent( win , win->pic_mod) ;
-	}
+		/* Parent window is changed. => Reset transparent. */
 
+		#ifdef  __DEBUG
+			kik_debug_printf( "reparent notify for transparency\n") ;
+		#endif
+		set_transparent( win) ;
+	}
+	
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		notify_reparent_to_children( win->children[count]) ;
@@ -326,11 +372,19 @@ notify_property_to_children(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
-	if( win->is_transparent && x_root_pixmap_available( win->disp->display))
+	if( win->is_transparent)
 	{
-		update_pic_transparent( win) ;
+		/* Background image of desktop is changed. */
+
+		if( ! IS_INHERIT_TRANSPARENT(win))
+		{
+		#ifdef  __DEBUG
+			kik_debug_printf( "property notify for transparency\n") ;
+		#endif
+			set_transparent( win) ;
+		}
 	}
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
@@ -345,7 +399,7 @@ is_descendant_window(
 	Window  window
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	if( win->my_window == window)
 	{
@@ -377,7 +431,7 @@ total_min_width(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 	u_int  min_width ;
 
 	min_width = win->min_width + win->margin * 2 ;
@@ -398,7 +452,7 @@ total_min_height(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 	u_int  min_height ;
 
 	min_height = win->min_height + win->margin * 2 ;
@@ -419,7 +473,7 @@ total_base_width(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 	u_int  base_width ;
 
 	base_width = win->base_width + win->margin * 2 ;
@@ -440,7 +494,7 @@ total_base_height(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 	u_int  base_height ;
 
 	base_height = win->base_height + win->margin * 2 ;
@@ -461,7 +515,7 @@ total_width_inc(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 	u_int  width_inc ;
 
 	width_inc = win->width_inc ;
@@ -491,7 +545,7 @@ total_height_inc(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 	u_int  height_inc ;
 
 	height_inc = win->height_inc ;
@@ -562,23 +616,18 @@ x_window_init(
 	
 	win->my_window = None ;
 
-	win->drawable = None ;
-	win->buffer = None ;
-
 	win->xft_draw = NULL ;
 
 	win->gc = NULL ;
 
-	win->fg_color = 0 ;
-	win->bg_color = 0 ;
+	memset( &win->fg_color , 0 , sizeof(win->fg_color)) ;
+	memset( &win->bg_color , 0 , sizeof(win->bg_color)) ;
 
 	win->parent_window = None ;
 	win->parent = NULL ;
 	win->children = NULL ;
 	win->num_of_children = 0 ;
 
-	win->use_buffer = 0 ;
-
 	win->pic_mod = NULL ;
 
 	win->wall_picture_is_set = 0 ;
@@ -659,7 +708,7 @@ x_window_final(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
 #ifdef  __DEBUG
 	kik_debug_printf( "[deleting child windows]\n") ;
@@ -677,13 +726,8 @@ x_window_final(
 	
 	x_xic_deactivate( win) ;
 
-	if( win->buffer)
-	{
-		XFreePixmap( win->disp->display , win->buffer) ;
-	}
-
 #ifdef  USE_TYPE_XFT
-	XftDrawDestroy( win->xft_draw) ;
+	x_window_set_xft( win , 0) ;
 #endif
 
 	if( win->create_gc)
@@ -701,6 +745,35 @@ x_window_final(
 	return  1 ;
 }
 
+/*
+ * Call this function in window_realized event at first.
+ */
+int
+x_window_set_xft(
+	x_window_t *  win ,
+	int  use_xft
+	)
+{
+#ifdef  USE_TYPE_XFT
+	if( use_xft && ! win->xft_draw)
+	{
+		win->xft_draw = XftDrawCreate( win->disp->display , win->my_window ,
+					win->disp->visual , win->disp->colormap) ;
+
+		return  1 ;
+	}
+	else if( ! use_xft && win->xft_draw)
+	{
+		XftDrawDestroy( win->xft_draw) ;
+		win->xft_draw = NULL ;
+
+		return  1 ;
+	}
+#endif
+
+	return  0 ;
+}
+
 int
 x_window_init_event_mask(
 	x_window_t *  win ,
@@ -791,15 +864,6 @@ x_window_set_wall_picture(
 	Pixmap  pic
 	)
 {
-	if( win->drawable != win->my_window)
-	{
-		/*
-		 * wall picture can not be used in double buffering mode.
-		 */
-
-		return  0 ;
-	}
-
 	if( win->is_transparent)
 	{
 		/*
@@ -818,7 +882,6 @@ x_window_set_wall_picture(
 	}
 
 	XSetWindowBackgroundPixmap( win->disp->display , win->my_window , pic) ;
-
 	win->wall_picture_is_set = 1 ;
 
 	if( win->window_exposed)
@@ -837,21 +900,23 @@ x_window_unset_wall_picture(
 	x_window_t *  win
 	)
 {
-	if( win->drawable != win->my_window)
-	{
-		/*
-		 * wall picture can not be used in double buffering mode.
-		 */
-
-		return  0 ;
-	}
-
+	/*
+	 * win->wall_picture_is_set == 0 doesn't mean that following codes
+	 * to disable wall picture is already processed.
+	 * e.g.) If x_window_unset_transparent() is called after
+	 * x_window_set_transparent() before my_window is created,
+	 * XSetWindowBackground() which is not called in x_window_set_bg_color()
+	 * (because is_transparent flag is set by x_window_set_transparent())
+	 * is never called in startup by checking win->wall_picture_is_set as follows.
+	 */
+#if  0
 	if( ! win->wall_picture_is_set)
 	{
 		/* already unset */
 
 		return  1 ;
 	}
+#endif
 
 	if( win->is_transparent)
 	{
@@ -874,7 +939,7 @@ x_window_unset_wall_picture(
 	}
 
 	XSetWindowBackgroundPixmap( win->disp->display , win->my_window , None) ;
-	XSetWindowBackground( win->disp->display , win->my_window , win->bg_color) ;
+	XSetWindowBackground( win->disp->display , win->my_window , win->bg_color.pixel) ;
 
 	win->wall_picture_is_set = 0 ;
 
@@ -889,17 +954,17 @@ x_window_unset_wall_picture(
 	return  1 ;
 }
 
+/*
+ * This function is possible to be called before my_window is created.
+ * (because x_screen_t doesn't contain transparent flag.)
+ */
 int
 x_window_set_transparent(
-	x_window_t *  win ,
+	x_window_t *  win ,		/* Transparency is applied to all children recursively */
 	x_picture_modifier_t *  pic_mod
 	)
 {
-	Window  parent ;
-	Window  root ;
-	Window *  list ;
-	u_int  n ;
-	int  count ;
+	u_int  count ;
 
 	win->pic_mod = pic_mod ;
 
@@ -912,34 +977,38 @@ x_window_set_transparent(
 		 */
 
 		win->is_transparent = 1 ;
-
-		goto  end ;
-	}
-
-	if( win->pic_mod || x_root_pixmap_available( win->disp->display))
-	{
-		update_pic_transparent( win) ;
 	}
 	else
 	{
-		set_transparent( win , ParentRelative) ;
+		set_transparent( win) ;
+	}
 
-		XSetWindowBackgroundPixmap( win->disp->display , win->my_window , ParentRelative) ;
+	for( count = 0 ; count < win->num_of_children ; count ++)
+	{
+		x_window_set_transparent( win->children[count] , win->pic_mod) ;
+	}
 
-		parent = win->my_window ;
+	return  1 ;
+}
 
-		while( 1)
-		{
-			XQueryTree( win->disp->display , parent , &root , &parent , &list , &n) ;
-			XFree(list) ;
+/*
+ * This function is possible to be called before my_window is created.
+ * (because x_screen_t doesn't contain transparent flag.)
+ */
+int
+x_window_unset_transparent(
+	x_window_t *  win
+	)
+{
+	u_int  count ;
 
-			if( parent == DefaultRootWindow(win->disp->display))
-			{
-				break ;
-			}
-
-			XSetWindowBackgroundPixmap( win->disp->display , parent , ParentRelative) ;
-		}
+	if( win->my_window == None)
+	{
+		win->is_transparent = 0 ;
+	}
+	else if( win->is_transparent)
+	{
+		unset_transparent( win) ;
 
 		if( win->window_exposed)
 		{
@@ -950,65 +1019,6 @@ x_window_set_transparent(
 		}
 	}
 
-end:
-	for( count = 0 ; count < win->num_of_children ; count ++)
-	{
-		x_window_set_transparent( win->children[count] , win->pic_mod) ;
-	}
-
-	return  1 ;
-}
-
-int
-x_window_unset_transparent(
-	x_window_t *  win
-	)
-{
-	int  count ;
-#if  0
-	Window  parent ;
-	Window  root ;
-	Window *  list ;
-	int  n ;
-#endif
-
-	if( ! win->is_transparent)
-	{
-		/* already unset */
-
-		return  1 ;
-	}
-
-	unset_transparent( win) ;
-
-#if  0
-	/*
-	 * XXX
-	 * is this necessary ?
-	 */
-	parent = win->my_window ;
-
-	for( count = 0 ; count < 2 ; count ++)
-	{
-		XQueryTree( win->disp->display , parent , &root , &parent , &list , &n) ;
-		XFree(list) ;
-		if( parent == DefaultRootWindow(win->disp->display))
-		{
-			break ;
-		}
-
-		XSetWindowBackgroundPixmap( win->disp->display , parent , None) ;
-	}
-#endif
-
-	if( win->window_exposed)
-	{
-	#if  0
-		x_window_clear_all( win) ;
-	#endif
-		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
-	}
-
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
 		x_window_unset_transparent( win->children[count]) ;
@@ -1017,25 +1027,6 @@ x_window_unset_transparent(
 	return  1 ;
 }
 
-/*
- * Double buffering.
- * If double buffering is used or not is not changeable after x_window_show().
- */
-int
-x_window_use_buffer(
-	x_window_t *  win
-	)
-{
-	win->use_buffer = 1 ;
-
-	win->event_mask &= ~VisibilityChangeMask ;
-
-	/* always true */
-	win->is_scrollable = 1 ;
-
-	return  1 ;
-}
-
 /*
  * Cursor is not changeable after x_window_show().
  */
@@ -1056,7 +1047,12 @@ x_window_set_fg_color(
 	x_color_t *  fg_color
 	)
 {
-	win->fg_color = fg_color->pixel ;
+	if( win->fg_color.pixel == fg_color->pixel)
+	{
+		return  0 ;
+	}
+	
+	win->fg_color = *fg_color ;
 
 	return  1 ;
 }
@@ -1067,11 +1063,16 @@ x_window_set_bg_color(
 	x_color_t *  bg_color
 	)
 {
-	win->bg_color = bg_color->pixel ;
+	if( win->bg_color.pixel == bg_color->pixel)
+	{
+		return  0 ;
+	}
+	
+	win->bg_color = *bg_color ;
 	
 	if( ! win->is_transparent && ! win->wall_picture_is_set)
 	{
-		XSetWindowBackground( win->disp->display , win->my_window , win->bg_color) ;
+		XSetWindowBackground( win->disp->display , win->my_window , win->bg_color.pixel) ;
 	}
 
 	return  1 ;
@@ -1143,10 +1144,10 @@ x_window_get_bg_gc(
 	x_window_t *  win
 	)
 {
-	x_gc_set_fg_color((win)->gc,(win)->bg_color) ;
+	x_gc_set_fg_color((win)->gc,(win)->bg_color.pixel) ;
 
 #if  0
-	x_gc_set_bg_color((win)->gc,(win)->fg_color) ;
+	x_gc_set_bg_color((win)->gc,(win)->fg_color.pixel) ;
 #endif
 
 	return  win->gc->gc ;
@@ -1158,7 +1159,8 @@ x_window_show(
 	int  hint
 	)
 {
-	int  count ;
+	u_int  count ;
+	XSetWindowAttributes  s_attr ;
 
 	if( win->my_window)
 	{
@@ -1174,23 +1176,6 @@ x_window_show(
 		win->parent_window = win->parent->my_window ;
 	}
 	
-	if( win->create_gc)
-	{
-		x_gc_t *  gc ;
-
-		if( ( gc = x_gc_new( win->disp->display)) == NULL)
-		{
-		#ifdef  DEBUG
-			kik_debug_printf( KIK_DEBUG_TAG " x_gc_new failed.\n") ;
-		#endif
-			win->create_gc = 0 ;
-		}
-		else
-		{
-			win->gc = gc ;
-		}
-	}
-
 	if( hint & XNegative)
 	{
 		win->x += (DisplayWidth( win->disp->display , win->disp->screen)
@@ -1203,22 +1188,37 @@ x_window_show(
 				- ACTUAL_HEIGHT(win)) ;
 	}
 
+	s_attr.background_pixel = win->bg_color.pixel ;
+	s_attr.border_pixel = win->fg_color.pixel ;
+	s_attr.colormap = win->disp->colormap ;
+#if  1
+	win->my_window = XCreateWindow(
+				win->disp->display , win->parent_window ,
+				win->x , win->y , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
+				0 , win->disp->depth , InputOutput , win->disp->visual ,
+				CWBackPixel | CWBorderPixel | CWColormap , &s_attr) ;
+#else
 	win->my_window = XCreateSimpleWindow(
-		win->disp->display , win->parent_window ,
-		win->x , win->y , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
-		0 , win->fg_color , win->bg_color) ;
+				win->disp->display , win->parent_window ,
+				win->x , win->y , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
+				0 , win->fg_color.pixel , win->bg_color.pixel) ;
+#endif
 
-	if( win->use_buffer)
+	if( win->create_gc)
 	{
-		win->drawable = win->buffer = XCreatePixmap( win->disp->display ,
-				win->parent_window ,
-				ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
-				DefaultDepth( win->disp->display , win->disp->screen)) ;
-	}
-	else
-	{
-		win->drawable = win->my_window ;
-		win->buffer = 0 ;
+		x_gc_t *  gc ;
+
+		if( ( gc = x_gc_new( win->disp->display , win->my_window)) == NULL)
+		{
+		#ifdef  DEBUG
+			kik_debug_printf( KIK_DEBUG_TAG " x_gc_new failed.\n") ;
+		#endif
+			win->create_gc = 0 ;
+		}
+		else
+		{
+			win->gc = gc ;
+		}
 	}
 
 	if( win->cursor_shape)
@@ -1231,14 +1231,11 @@ x_window_show(
 		}
 	}
 
-#ifdef  USE_TYPE_XFT
-	win->xft_draw = XftDrawCreate( win->disp->display , win->drawable ,
-					DefaultVisual( win->disp->display , win->disp->screen) ,
-					DefaultColormap( win->disp->display , win->disp->screen)) ;
-#endif
-
-	if( win->parent == NULL)
+	/* Don't use win->parent here in case mlterm works as libvte. */
+	if( PARENT_WINDOWID_IS_TOP(win))
 	{
+		/* Root window */
+
 		XSizeHints  size_hints ;
 		XClassHint  class_hint ;
 		XWMHints  wm_hints ;
@@ -1419,25 +1416,13 @@ x_window_resize(
 	win->width = width ;
 	win->height = height ;
 
-	if( win->buffer)
-	{
-		XFreePixmap( win->disp->display , win->buffer) ;
-		win->buffer = XCreatePixmap( win->disp->display ,
-			win->parent_window , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
-			DefaultDepth( win->disp->display , win->disp->screen)) ;
-		win->drawable = win->buffer ;
-
-	#ifdef  USE_TYPE_XFT
-		XftDrawChange( win->xft_draw , win->drawable) ;
-	#endif
-	}
-
 	if( (flag & NOTIFY_TO_PARENT) && win->parent && win->parent->child_window_resized)
 	{
 		(*win->parent->child_window_resized)( win->parent , win) ;
 	}
 
-	XResizeWindow( win->disp->display , win->my_window , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+	XResizeWindow( win->disp->display , win->my_window ,
+		ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
 
 	if( (flag & NOTIFY_TO_MYSELF) && win->window_resized)
 	{
@@ -1460,7 +1445,15 @@ x_window_resize_with_margin(
 	x_event_dispatch_t  flag	/* NOTIFY_TO_PARENT , NOTIFY_TO_MYSELF */
 	)
 {
-	return  x_window_resize( win , width - win->margin * 2 , height - win->margin * 2 , flag) ;
+	u_int  min_width ;
+	u_int  min_height ;
+
+	min_width = total_min_width( win) ;
+	min_height = total_min_height( win) ;
+	
+	return  x_window_resize( win ,
+			width <= min_width ? min_width : width - win->margin * 2 ,
+			height <= min_height ? min_height : height - win->margin * 2 , flag) ;
 }
 
 int
@@ -1608,21 +1601,17 @@ x_window_clear(
 	u_int	height
 	)
 {
-	if( win->drawable == win->buffer)
+#ifdef  USE_TYPE_XFT
+	if( ! win->wall_picture_is_set && win->xft_draw)
 	{
-		x_gc_set_fg_color(win->gc, win->bg_color) ;
-		
-		XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
 			x + win->margin , y + win->margin , width , height) ;
 	}
-	else if( win->drawable == win->my_window)
-	{
-		XClearArea( win->disp->display , win->drawable , x + win->margin , y + win->margin ,
-			width , height , 0) ;
-	}
 	else
+#endif
 	{
-		return  0 ;
+		XClearArea( win->disp->display , win->my_window ,
+			x + win->margin , y + win->margin , width , height , False) ;
 	}
 
 	return  1 ;
@@ -1637,33 +1626,31 @@ x_window_clear_margin_area(
 	{
 		return  1 ;
 	}
-	else if( win->drawable == win->buffer)
-	{
-		x_gc_set_fg_color(win->gc, win->bg_color) ;
 
-		XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
-			0 , 0 , win->margin , ACTUAL_HEIGHT(win)) ;
-		XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
-			win->margin , 0 , win->width , win->margin) ;
-		XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
-			win->width + win->margin , 0 , win->margin , ACTUAL_HEIGHT(win)) ;
-		XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
-			win->margin , win->height + win->margin , win->width , win->margin) ;
-	}
-	else if( win->drawable == win->my_window)
+#ifdef  USE_TYPE_XFT
+	if( ! win->wall_picture_is_set && win->xft_draw)
 	{
-		XClearArea( win->disp->display , win->drawable ,
-			0 , 0 , win->margin , ACTUAL_HEIGHT(win) , 0) ;
-		XClearArea( win->disp->display , win->drawable ,
-			win->margin , 0 , win->width , win->margin , 0) ;
-		XClearArea( win->disp->display , win->drawable ,
-			win->width + win->margin , 0 , win->margin , ACTUAL_HEIGHT(win) , 0) ;
-		XClearArea( win->disp->display , win->drawable ,
-			win->margin , win->height + win->margin , win->width , win->margin , 0) ;
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
+			0 , 0 , win->margin , ACTUAL_HEIGHT(win)) ;
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
+			win->margin , 0 , win->width , win->margin) ;
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
+			win->width + win->margin , 0 , win->margin , ACTUAL_HEIGHT(win)) ;
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
+			win->margin , win->height + win->margin ,
+			win->width , win->margin) ;
 	}
 	else
+#endif
 	{
-		return  0 ;
+		XClearArea( win->disp->display , win->my_window ,
+			0 , 0 , win->margin , ACTUAL_HEIGHT(win) , 0) ;
+		XClearArea( win->disp->display , win->my_window ,
+			win->margin , 0 , win->width , win->margin , 0) ;
+		XClearArea( win->disp->display , win->my_window , win->width + win->margin ,
+			0 , win->margin , ACTUAL_HEIGHT(win) , 0) ;
+		XClearArea( win->disp->display , win->my_window , win->margin ,
+			win->height + win->margin , win->width , win->margin , 0) ;
 	}
 
 	return  1 ;
@@ -1674,21 +1661,17 @@ x_window_clear_all(
 	x_window_t *  win
 	)
 {
-	if( win->drawable == win->buffer)
+#ifdef  USE_TYPE_XFT
+	if( ! win->wall_picture_is_set && win->xft_draw)
 	{
-		x_gc_set_fg_color(win->gc, win->bg_color) ;
-		
-		XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->bg_color) ,
 			0 , 0 , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
 	}
-	else if( win->drawable == win->my_window)
-	{
-		XClearArea( win->disp->display , win->drawable , 0 , 0 ,
-			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) , 0) ;
-	}
 	else
+#endif
 	{
-		return  0 ;
+		XClearArea( win->disp->display , win->my_window , 0 , 0 ,
+			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) , 0) ;
 	}
 
 	return  1 ;
@@ -1703,10 +1686,20 @@ x_window_fill(
 	u_int	height
 	)
 {
-	restore_fg_color( win) ;
-	
-	XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
-		x + win->margin , y + win->margin , width , height) ;
+#ifdef  USE_TYPE_XFT
+	if( win->xft_draw)
+	{
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
+			x + win->margin , y + win->margin , width , height) ;
+	}
+	else
+#endif
+	{
+		restore_fg_color( win) ;
+
+		XFillRectangle( win->disp->display , win->my_window , win->gc->gc ,
+			x + win->margin , y + win->margin , width , height) ;
+	}
 
 	return  1 ;
 }
@@ -1721,10 +1714,20 @@ x_window_fill_with(
 	u_int	height
 	)
 {
-	x_gc_set_fg_color( win->gc, color->pixel) ;
+#ifdef  USE_TYPE_XFT
+	if( win->xft_draw)
+	{
+		XftDrawRect( win->xft_draw , x_color_to_xft( color) ,
+			x + win->margin , y + win->margin , width , height) ;
+	}
+	else
+#endif
+	{
+		x_gc_set_fg_color( win->gc, color->pixel) ;
 
-	XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
-		x + win->margin , y + win->margin , width , height) ;
+		XFillRectangle( win->disp->display , win->my_window , win->gc->gc ,
+			x + win->margin , y + win->margin , width , height) ;
+	}
 
 	return  1 ;
 }
@@ -1734,10 +1737,20 @@ x_window_blank(
 	x_window_t *  win
 	)
 {
-	restore_fg_color( win) ;
-	
-	XFillRectangle( win->disp->display , win->drawable , win->gc->gc , 0 , 0 ,
-		ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+#ifdef  USE_TYPE_XFT
+	if( win->xft_draw)
+	{
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) , 0 , 0 ,
+			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+	}
+	else
+#endif
+	{
+		restore_fg_color( win) ;
+
+		XFillRectangle( win->disp->display , win->my_window , win->gc->gc , 0 , 0 ,
+			ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
+	}
 
 	return  1 ;
 }
@@ -1755,7 +1768,7 @@ x_window_blank_with(
 {
 	x_gc_set_fg_color( win->gc, color->pixel) ;
 
-	XFillRectangle( win->disp->display , win->drawable , win->gc->gc ,
+	XFillRectangle( win->disp->display , win->my_window , win->gc->gc ,
 		0 , 0 , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win)) ;
 
 	return  1 ;
@@ -1778,7 +1791,7 @@ x_window_idling(
 	x_window_t *  win
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
@@ -1808,7 +1821,7 @@ x_window_receive_event(
 	XEvent *  event
 	)
 {
-	int  count ;
+	u_int  count ;
 
 	for( count = 0 ; count < win->num_of_children ; count ++)
 	{
@@ -1817,6 +1830,7 @@ x_window_receive_event(
 			return  1 ;
 		}
 	}
+
 	if( win->my_window != event->xany.window)
 	{
 		/*
@@ -1837,11 +1851,20 @@ x_window_receive_event(
 			}
 		}
 
-		if( x_root_pixmap_available( win->disp->display) &&
-			( event->type == PropertyNotify) &&
-			( win == x_get_root_window( win)) &&
-			( event->xproperty.atom == XA_XROOTPMAP_ID(win->disp->display)))
+		if( event->type == PropertyNotify &&
+			win == x_get_root_window( win) &&
+			(event->xproperty.atom == XA_XSETROOT_ID(win->disp->display) ||
+			 event->xproperty.atom == XA_XROOTPMAP_ID(win->disp->display)) )
 		{
+			/*
+			 * Background image is changed.
+			 * (notify_property_to_children() is called here because
+			 * event->xproperty.window is not win->my_window.)
+			 *
+			 * twm => XA_XSETROOT_ID
+			 * englightment => XA_XROOTPMAP_ID
+			 */
+			 
 			notify_property_to_children( win) ;
 
 			return  1 ;
@@ -1867,10 +1890,10 @@ x_window_receive_event(
 				(*win->mapping_notify)( win);
 			}
 		}
-		
+
 		return  0 ;
 	}
-	
+
 #ifndef  DISABLE_XDND
 	if( x_dnd_filter_event( event, win))
 	{
@@ -2008,6 +2031,12 @@ x_window_receive_event(
 			win->button_is_pressing = 1 ;
 			win->prev_button_press_event = event->xbutton ;
 		}
+
+		if( ! win->is_focused)
+		{
+			XSetInputFocus( win->disp->display , win->my_window ,
+				RevertToParent , CurrentTime) ;
+		}
 	}
 	else if( event->type == Expose /* && event->xexpose.count == 0 */)
 	{
@@ -2078,12 +2107,9 @@ x_window_receive_event(
 			}
 		}
 
-		if( ! restore_view( win , x , y , width , height))
+		if( win->window_exposed)
 		{
-			if( win->window_exposed)
-			{
-				(*win->window_exposed)( win , x , y , width , height) ;
-			}
+			(*win->window_exposed)( win , x , y , width , height) ;
 		}
 	}
 	else if( event->type == ConfigureNotify)
@@ -2091,6 +2117,15 @@ x_window_receive_event(
 		int  is_changed ;
 		XEvent  next_ev ;
 
+		/*
+		 * Optimize transparent processing in notify_configure_to_children.
+		 */
+		while( XCheckTypedWindowEvent( win->disp->display , win->my_window ,
+			ConfigureNotify , &next_ev))
+		{
+			*event = next_ev ;
+		}
+
 		is_changed = 0 ;
 
 		if( event->xconfigure.x != win->x || event->xconfigure.y != win->y)
@@ -2126,20 +2161,6 @@ x_window_receive_event(
 			win->width = event->xconfigure.width - win->margin * 2 ;
 			win->height = event->xconfigure.height - win->margin * 2 ;
 
-			if( win->buffer)
-			{
-				XFreePixmap( win->disp->display , win->buffer) ;
-				win->buffer = XCreatePixmap( win->disp->display ,
-					win->parent_window ,
-					ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
-					DefaultDepth( win->disp->display , win->disp->screen)) ;
-				win->drawable = win->buffer ;
-
-			#ifdef  USE_TYPE_XFT
-				XftDrawChange( win->xft_draw , win->drawable) ;
-			#endif
-			}
-
 		#if  0
 			x_window_clear_all( win) ;
 		#endif
@@ -2152,21 +2173,6 @@ x_window_receive_event(
 			is_changed = 1 ;
 		}
 
-		/*
-		 * transparent processing.
-		 */
-
-		if( is_changed &&
-			XCheckMaskEvent( win->disp->display , StructureNotifyMask , &next_ev))
-		{
-			if( next_ev.type == ConfigureNotify)
-			{
-				is_changed = 0 ;
-			}
-
-			XPutBackEvent( win->disp->display , &next_ev) ;
-		}
-
 		if( is_changed)
 		{
 			notify_configure_to_children( win) ;
@@ -2174,19 +2180,31 @@ x_window_receive_event(
 	}
 	else if( event->type == ReparentNotify)
 	{
+		XEvent  next_ev ;
+		
 		/*
-		 * transparent processing.
+		 * Optimize transparent processing in notify_reparent_to_children.
 		 */
+		while( XCheckTypedWindowEvent( win->disp->display , win->my_window ,
+			ReparentNotify , &next_ev))
+		{
+			*event = next_ev ;
+		}
 
 		win->x = event->xreparent.x ;
 		win->y = event->xreparent.y ;
 
 		notify_reparent_to_children( win) ;
 	}
+#if  0
 	else if( event->type == MapNotify)
 	{
-		notify_reparent_to_children( win) ;
+		if( win->is_transparent && ! win->wall_picture_is_set)
+		{
+			set_transparent( win) ;
+		}
 	}
+#endif
 	else if( event->type == SelectionClear)
 	{
 		/*
@@ -2414,8 +2432,9 @@ x_window_receive_event(
 				exit(0) ;
 			}
 		}
-#if 0
-		if( event->xclient.format == 32 &&
+		
+#if  0
+		else if( event->xclient.format == 32 &&
 			event->xclient.data.l[0] == XA_TAKE_FOCUS( win->disp->display))
 		{
 			kik_warn_printf( KIK_DEBUG_TAG
@@ -2569,7 +2588,7 @@ x_window_scroll_upward_region(
 		return  0 ;
 	}
 
-	XCopyArea( win->disp->display , win->drawable , win->drawable , win->gc->gc ,
+	XCopyArea( win->disp->display , win->my_window , win->my_window , win->gc->gc ,
 		win->margin , win->margin + boundary_start + height ,	/* src */
 		win->width , boundary_end - boundary_start - height ,	/* size */
 		win->margin , win->margin + boundary_start) ;		/* dst */
@@ -2610,7 +2629,7 @@ x_window_scroll_downward_region(
 		return  0 ;
 	}
 
-	XCopyArea( win->disp->display , win->drawable , win->drawable , win->gc->gc ,
+	XCopyArea( win->disp->display , win->my_window , win->my_window , win->gc->gc ,
 		win->margin , win->margin + boundary_start ,
 		win->width , boundary_end - boundary_start - height ,
 		win->margin , win->margin + boundary_start + height) ;
@@ -2651,7 +2670,7 @@ x_window_scroll_leftward_region(
 		return  0 ;
 	}
 
-	XCopyArea( win->disp->display , win->drawable , win->drawable , win->gc->gc ,
+	XCopyArea( win->disp->display , win->my_window , win->my_window , win->gc->gc ,
 		win->margin + boundary_start + width , win->margin ,	/* src */
 		boundary_end - boundary_start - width , win->height ,	/* size */
 		win->margin + boundary_start , win->margin) ;		/* dst */
@@ -2691,7 +2710,7 @@ x_window_scroll_rightward_region(
 		return  0 ;
 	}
 
-	XCopyArea( win->disp->display , win->drawable , win->drawable , win->gc->gc ,
+	XCopyArea( win->disp->display , win->my_window , win->my_window , win->gc->gc ,
 		win->margin + boundary_start , win->margin ,
 		boundary_end - boundary_start - width , win->height ,
 		win->margin + boundary_start + width , win->margin) ;
@@ -2715,7 +2734,7 @@ x_window_draw_decsp_string(
 		x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 
 		return  x_decsp_font_draw_image_string( font->decsp_font , win->disp->display ,
-				win->drawable , win->gc->gc , x + win->margin , y + win->margin ,
+				win->my_window , win->gc->gc , x + win->margin , y + win->margin ,
 				str , len) ;
 	}
 #ifdef  USE_TYPE_XCORE
@@ -2748,7 +2767,7 @@ x_window_draw_decsp_image_string(
 		x_gc_set_bg_color( win->gc, bg_color->pixel) ;
 
 		return  x_decsp_font_draw_image_string( font->decsp_font , win->disp->display ,
-				win->drawable , win->gc->gc , x + win->margin , y + win->margin ,
+				win->my_window , win->gc->gc , x + win->margin , y + win->margin ,
 				str , len) ;
 	}
 #ifdef  USE_TYPE_XCORE
@@ -2779,13 +2798,13 @@ x_window_draw_string(
 	x_gc_set_fid( win->gc, font->xfont->fid) ;
 	x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 
-	XDrawString( win->disp->display , win->drawable , win->gc->gc ,
+	XDrawString( win->disp->display , win->my_window , win->gc->gc ,
 		x + (font->is_var_col_width ? 0 : font->x_off) + win->margin ,
 		y + win->margin , (char *)str , len) ;
 
 	if( font->is_double_drawing)
 	{
-		XDrawString( win->disp->display , win->drawable , win->gc->gc ,
+		XDrawString( win->disp->display , win->my_window , win->gc->gc ,
 			x + (font->is_var_col_width ? 0 : font->x_off) + win->margin + 1 ,
 			y + win->margin , (char *)str , len) ;
 	}
@@ -2807,13 +2826,13 @@ x_window_draw_string16(
 	x_gc_set_fid( win->gc, font->xfont->fid) ;
 	x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 
-	XDrawString16( win->disp->display , win->drawable , win->gc->gc ,
+	XDrawString16( win->disp->display , win->my_window , win->gc->gc ,
 		       x + (font->is_var_col_width ? 0 : font->x_off) + win->margin ,
 		       y + win->margin , str , len) ;
 
 	if( font->is_double_drawing)
 	{
-		XDrawString16( win->disp->display , win->drawable , win->gc->gc ,
+		XDrawString16( win->disp->display , win->my_window , win->gc->gc ,
 			       x + (font->is_var_col_width ? 0 : font->x_off) + win->margin + 1 ,
 			       y + win->margin , str , len) ;
 	}
@@ -2837,13 +2856,13 @@ x_window_draw_image_string(
 	x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 	x_gc_set_bg_color( win->gc, bg_color->pixel) ;
 
-	XDrawImageString( win->disp->display , win->drawable , win->gc->gc ,
+	XDrawImageString( win->disp->display , win->my_window , win->gc->gc ,
 			  x + (font->is_var_col_width ? 0 : font->x_off) + win->margin ,
 			  y + win->margin , (char *)str , len) ;
 
 	if( font->is_double_drawing)
 	{
-		XDrawString( win->disp->display , win->drawable , win->gc->gc ,
+		XDrawString( win->disp->display , win->my_window , win->gc->gc ,
 			     x + (font->is_var_col_width ? 0 : font->x_off) + win->margin + 1 ,
 			     y + win->margin , (char *)str , len) ;
 	}
@@ -2867,13 +2886,13 @@ x_window_draw_image_string16(
 	x_gc_set_fg_color( win->gc, fg_color->pixel) ;
 	x_gc_set_bg_color( win->gc, bg_color->pixel) ;
 
-	XDrawImageString16( win->disp->display , win->drawable , win->gc->gc ,
+	XDrawImageString16( win->disp->display , win->my_window , win->gc->gc ,
 			    x + (font->is_var_col_width ? 0 : font->x_off) + win->margin ,
 			    y + win->margin , str , len) ;
 
 	if( font->is_double_drawing)
 	{
-		XDrawString16( win->disp->display , win->drawable , win->gc->gc ,
+		XDrawString16( win->disp->display , win->my_window , win->gc->gc ,
 			       x + (font->is_var_col_width ? 0 : font->x_off) + win->margin + 1 ,
 			       y + win->margin , str , len) ;
 	}
@@ -2895,16 +2914,16 @@ x_window_xft_draw_string8(
 	size_t  len
 	)
 {
-	XftColor  xftcolor ;
+	XftColor *  xftcolor ;
 
-	x_color_to_xft( &xftcolor , fg_color) ;
+	xftcolor = x_color_to_xft( fg_color) ;
 
-	XftDrawString8( win->xft_draw , &xftcolor , font->xft_font ,
+	XftDrawString8( win->xft_draw , xftcolor , font->xft_font ,
 		x + win->margin , y + win->margin , str , len) ;
 
 	if( font->is_double_drawing)
 	{
-		XftDrawString8( win->xft_draw , &xftcolor , font->xft_font ,
+		XftDrawString8( win->xft_draw , xftcolor , font->xft_font ,
 			x + win->margin + 1 , y + win->margin , str , len) ;
 	}
 
@@ -2922,16 +2941,16 @@ x_window_xft_draw_string32(
 	u_int  len
 	)
 {
-	XftColor  xftcolor ;
+	XftColor *  xftcolor ;
 
-	x_color_to_xft( &xftcolor , fg_color) ;
-	
-	XftDrawString32( win->xft_draw , &xftcolor , font->xft_font ,
+	xftcolor = x_color_to_xft( fg_color) ;
+
+	XftDrawString32( win->xft_draw , xftcolor , font->xft_font ,
 		x + win->margin , y + win->margin , str , len) ;
 
 	if( font->is_double_drawing)
 	{
-		XftDrawString32( win->xft_draw , &xftcolor , font->xft_font ,
+		XftDrawString32( win->xft_draw , xftcolor , font->xft_font ,
 			x + win->margin + 1 , y + win->margin , str , len) ;
 	}
 
@@ -2948,12 +2967,35 @@ x_window_draw_rect_frame(
 	int  y2
 	)
 {
-	restore_fg_color(win) ;
-	
-	XDrawLine( win->disp->display , win->drawable , win->gc->gc , x1 , y1 , x2 , y1) ;
-	XDrawLine( win->disp->display , win->drawable , win->gc->gc , x1 , y1 , x1 , y2) ;
-	XDrawLine( win->disp->display , win->drawable , win->gc->gc , x2 , y2 , x2 , y1) ;
-	XDrawLine( win->disp->display , win->drawable , win->gc->gc , x2 , y2 , x1 , y2) ;
+#ifdef  USE_TYPE_XFT
+	if( win->xft_draw)
+	{
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
+			x1 , y1 , 1 , y2 - y1 + 1) ;
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
+			x1 , y1 , x2 - x1 + 1 , 1) ;
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
+			x1 , y2 , x2 - x1 + 1 , 1) ;
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
+			x2 , y1 , 1 , y2 - y1 + 1) ;
+	}
+	else
+#endif
+	{
+		XPoint  points[5] =
+		{
+			{ x1 , y1 } ,
+			{ x1 , y2 } ,
+			{ x2 , y2 } ,
+			{ x2 , y1 } ,
+			{ x1 , y1 } ,
+		} ;
+
+		restore_fg_color(win) ;
+
+		XDrawLines( win->disp->display , win->my_window , win->gc->gc , points , 5 ,
+			CoordModeOrigin) ;
+	}
 
 	return  1 ;
 }
@@ -2967,9 +3009,19 @@ x_window_draw_line(
 	int  y2
 	)
 {
-	restore_fg_color(win) ;
-	
-	XDrawLine( win->disp->display, win->drawable, win->gc->gc, x1, y1, x2, y2) ;
+#ifdef  USE_TYPE_XFT
+	if( win->xft_draw)
+	{
+		XftDrawRect( win->xft_draw , x_color_to_xft( &win->fg_color) ,
+			x1 , y1 , x2 - x1 + 1 , y2 - y1 + 1) ;
+	}
+	else
+#endif
+	{
+		restore_fg_color(win) ;
+
+		XDrawLine( win->disp->display , win->my_window , win->gc->gc , x1 , y1 , x2 , y2) ;
+	}
 
 	return  1 ;
 }
@@ -3347,8 +3399,8 @@ x_window_get_mod_ignore_mask(
 	int  count ;
 	u_int  ignore ;
 	u_int  masks[] = { Mod1Mask , Mod2Mask , Mod3Mask , Mod4Mask , Mod5Mask } ;
-	KeySym default_keysyms[] = { XK_Scroll_Lock, XK_ISO_Level3_Lock,
-				     NoSymbol} ;
+	KeySym default_keysyms[] = { XK_Num_Lock , XK_Scroll_Lock , XK_ISO_Level3_Lock ,
+				     NoSymbol } ;
 
 	if( !keysyms)
 	{
@@ -3554,7 +3606,7 @@ x_window_paste(
 		kik_warn_printf( KIK_DEBUG_TAG " height is modified -> %d\n" , src_height) ;
 	}
 
-	XCopyArea( win->disp->display , src , win->drawable , win->gc->gc ,
+	XCopyArea( win->disp->display , src , win->my_window , win->gc->gc ,
 		src_x , src_y , src_width , src_height ,
 		dst_x + win->margin , dst_y + win->margin) ;
 
