@@ -29,6 +29,8 @@
 #define  ACTUAL_WIDTH(win)  ((win)->width + (win)->margin * 2)
 #define  ACTUAL_HEIGHT(win)  ((win)->height + (win)->margin * 2)
 
+#define  PARENT_WINDOWID_IS_TOP(win)  ((win)->parent_window == (win)->disp->my_window)
+
 
 typedef enum  x_event_dispatch
 {
@@ -68,8 +70,6 @@ typedef struct  x_window
 	x_display_t *  disp ;
 	
 	Window  my_window ;
-	Drawable  drawable ;
-	Pixmap  buffer ;
 
 #ifdef  USE_TYPE_XFT
 	XftDraw *  xft_draw ;
@@ -77,8 +77,8 @@ typedef struct  x_window
 	void *  xft_draw ; /* dummy to keep the size of x_window_t */
 #endif
 
-	u_long  fg_color ;
-	u_long  bg_color ;
+	x_color_t  fg_color ;
+	x_color_t  bg_color ;
 
 	x_gc_t *  gc ;
 
@@ -141,8 +141,7 @@ typedef struct  x_window
 	 */
 
 	int8_t  is_sel_owner ;
-	int8_t  use_buffer ;
-	int8_t  wall_picture_is_set ;
+	int8_t  wall_picture_is_set ;	/* Actually set picture (including transparency) or not. */
 	int8_t  is_transparent ;
 	int8_t  is_scrollable ;
 	int8_t  is_focused ;
@@ -180,6 +179,7 @@ typedef struct  x_window
 #endif
 	void (*window_deleted)( struct x_window *) ;
 	void (*mapping_notify)( struct x_window *) ;
+
 } x_window_t ;
 
 
@@ -190,6 +190,18 @@ int  x_window_init( x_window_t *  win ,
 
 int  x_window_final( x_window_t *  win) ;
 
+int  x_window_set_xft( x_window_t *  win , int  use_xft) ;
+
+int  x_window_init_event_mask( x_window_t *  win , long  event_mask) ;
+
+int  x_window_add_event_mask( x_window_t *  win , long  event_mask) ;
+
+int  x_window_remove_event_mask( x_window_t *  win , long  event_mask) ;
+
+/* int  x_window_grab_pointer( x_window_t *  win) ; */
+
+int  x_window_ungrab_pointer( x_window_t *  win) ;
+
 int  x_window_set_wall_picture( x_window_t *  win , Pixmap  pic) ;
 
 int  x_window_unset_wall_picture( x_window_t *  win) ;
@@ -198,8 +210,6 @@ int  x_window_set_transparent( x_window_t *  win , x_picture_modifier_ptr_t  pic
 
 int  x_window_unset_transparent( x_window_t *  win) ;
 
-int  x_window_use_buffer( x_window_t *  win) ;
-
 int  x_window_set_cursor( x_window_t *  win , u_int  cursor_shape) ;
 
 int  x_window_set_fg_color( x_window_t *  win , x_color_t *  fg_color) ;
@@ -214,16 +224,6 @@ GC  x_window_get_fg_gc( x_window_t *  win) ;
 
 GC  x_window_get_bg_gc( x_window_t *  win) ;
 
-int  x_window_init_event_mask( x_window_t *  win , long  event_mask) ;
-
-int  x_window_add_event_mask( x_window_t *  win , long  event_mask) ;
-
-int  x_window_remove_event_mask( x_window_t *  win , long  event_mask) ;
-
-/* int  x_window_grab_pointer( x_window_t *  win) ; */
-
-int  x_window_ungrab_pointer( x_window_t *  win) ;
-
 int  x_window_show( x_window_t *  win , int  hint) ;
 
 int  x_window_map( x_window_t *  win) ;
