@@ -2,12 +2,6 @@
  *	$Id$
  */
 
-/*
- * Functions designed and implemented by Minami Hirokazu(minami@mistfall.net) are:
- * - XDND support
- * - Extended Window Manager Hint(Icon) support
- */
-
 #include  "x_window.h"
 
 #include  <stdlib.h>		/* abs */
@@ -54,79 +48,6 @@ static mkf_parser_t *  m_cp_parser ;
 
 /* --- static functions --- */
 
-#ifndef  USE_WIN32GUI
-/*
- * only for double buffering
- */
-static int
-restore_view(
-	x_window_t *  win ,
-	int  x ,
-	int  y ,
-	u_int	width ,
-	u_int	height
-	)
-{
-	if( win->buffer == win->drawable)
-	{
-		if( win->width < x + width)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG
-				" update size (x)%d (w)%d is overflowed.(screen width %d)\n" ,
-				x , width , win->width) ;
-		#endif
-
-			width = win->width - x ;
-
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " width is modified -> %d\n" , width) ;
-		#endif
-		}
-
-		if( win->height < y + height)
-		{
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG
-				" update size (y)%d (h)%d is overflowed.(screen height is %d)\n" ,
-				y , height , win->height) ;
-		#endif
-
-			height = win->height - y ;
-
-		#ifdef  DEBUG
-			kik_warn_printf( KIK_DEBUG_TAG " height is modified -> %d\n" , height) ;
-		#endif
-		}
-
-		XCopyArea( win->display , win->drawable , win->my_window , win->gc ,
-			x + win->margin , y + win->margin , width , height ,
-			x + win->margin , y + win->margin) ;
-
-		return  1 ;
-	}
-	else
-	{
-		return  0 ;
-	}
-}
-#endif
-
-#if  0
-/*
- * XXX
- * Not used for now.
- */
-static int
-restore_view_all(
-	x_window_t *  win
-	)
-{
-	return	restore_view( win , 0 , 0 , win->width , win->height) ;
-}
-#endif
-
-#ifdef  USE_WIN32GUI
 static int
 set_transparent(
 	x_window_t *  win ,
@@ -155,7 +76,7 @@ set_transparent(
 			SetLayeredWindowAttributes( win->my_window , 0 , pic_mod->alpha ,
 				LWA_ALPHA) ;
 		#else
-			SetLayeredWindowAttributes( win->my_window , win->bg_color ,
+			SetLayeredWindowAttributes( win->my_window , win->bg_color.pixel ,
 				0 , LWA_COLORKEY) ;
 		#endif
 		}
@@ -174,33 +95,6 @@ set_transparent(
 	return  0 ;
 #endif
 }
-#else	/* USE_WIN32GUI */
-static int
-set_transparent(
-	x_window_t *  win ,
-	Pixmap  pixmap
-	)
-{
-	/*
-	 * !! Notice !!
-	 * is_transparent must be on before x_window_set_wall_picture() because
-	 * x_window_set_wall_picture() doesn't do anything if is_transparent
-	 * flag is off.
-	 */
-	win->is_transparent = 0 ;
-
-	if( ! x_window_set_wall_picture( win , pixmap))
-	{
-		win->pic_mod = NULL ;
-
-		return  0 ;
-	}
-
-	win->is_transparent = 1 ;
-
-	return  1 ;
-}
-#endif
 
 static int
 unset_transparent(
@@ -240,20 +134,6 @@ unset_transparent(
 #else
 	return  0 ;
 #endif
-
-#ifndef  USE_WIN32GUI
-	/*
-	 * !! Notice !!
-	 * is_transparent must be off before x_window_unset_wall_picture() because
-	 * x_window_unset_wall_picture() doesn't do anything if is_transparent
-	 * flag is on.
-	 */
-	win->is_transparent = 0 ;
-	
-	win->pic_mod = NULL ;
-
-	return  x_window_unset_wall_picture( win) ;
-#endif
 }
 
 static int
@@ -293,11 +173,7 @@ check_scrollable(
 	x_window_t *  win	/* Assume root window. */
 	)
 {
-	if( win->use_buffer)
-	{
-		return  1 ;
-	}
-	else if( win->is_focused)
+	if( win->is_focused)
 	{
 	#if  0
 		kik_debug_printf( "SCREEN W %d H %d WINDOW W %d H %d X %d Y %d " ,
@@ -914,18 +790,13 @@ x_window_init(
 	win->parent_window = None ;
 	win->my_window = None ;
 
-	win->drawable = None ;
-	win->buffer = None ;
-
-	win->fg_color = 0 ;
-	win->bg_color = 0 ;
+	memset( &win->fg_color , 0 , sizeof(win->fg_color)) ;
+	memset( &win->bg_color , 0 , sizeof(win->bg_color)) ;
 
 	win->parent = NULL ;
 	win->children = NULL ;
 	win->num_of_children = 0 ;
 
-	win->use_buffer = 0 ;
-
 	win->pic_mod = NULL ;
 
 	win->wall_picture_is_set = 0 ;
@@ -977,7 +848,11 @@ x_window_init(
 	win->click_num = 0 ;
 	win->button_is_pressing = 0 ;
 	win->dnd = NULL ;
+#ifndef  UTF16_IME_CHAR
 	win->app_name = "mlterm" ;
+#else
+	win->app_name = L"mlterm" ;
+#endif
 
 	win->window_realized = NULL ;
 	win->window_finalized = NULL ;
@@ -1032,11 +907,6 @@ x_window_final(
 	 * If you want to close window, call SendMessage( WM_CLOSE ) instead of x_window_final().
 	 */
 #else
-	if( win->buffer)
-	{
-		XFreePixmap( win->display , win->buffer) ;
-	}
-
 	XDestroyWindow( win->display , win->my_window) ;
 #endif
 
@@ -1054,6 +924,15 @@ x_window_final(
 	return  1 ;
 }
 
+int
+x_window_set_xft(
+	x_window_t *  win ,
+	int  use_xft
+	)
+{
+	return  0 ;
+}
+
 int
 x_window_init_event_mask(
 	x_window_t *  win ,
@@ -1096,15 +975,6 @@ x_window_set_wall_picture(
 	)
 {
 #ifndef  USE_WIN32GUI
-	if( win->drawable != win->my_window)
-	{
-		/*
-		 * wall picture can not be used in double buffering mode.
-		 */
-
-		return  0 ;
-	}
-
 	if( win->is_transparent)
 	{
 		/*
@@ -1142,15 +1012,6 @@ x_window_unset_wall_picture(
 	)
 {
 #ifndef  USE_WIN32GUI
-	if( win->drawable != win->my_window)
-	{
-		/*
-		 * wall picture can not be used in double buffering mode.
-		 */
-
-		return  0 ;
-	}
-
 	if( ! win->wall_picture_is_set)
 	{
 		/* already unset */
@@ -1179,7 +1040,7 @@ x_window_unset_wall_picture(
 	}
 
 	XSetWindowBackgroundPixmap( win->display , win->my_window , None) ;
-	XSetWindowBackground( win->display , win->my_window , win->bg_color) ;
+	XSetWindowBackground( win->display , win->my_window , win->bg_color.pixel) ;
 
 	win->wall_picture_is_set = 0 ;
 
@@ -1205,70 +1066,6 @@ x_window_set_transparent(
 	}
 	
 	return  set_transparent( x_get_root_window( win) , pic_mod) ;
-
-#ifndef  USE_WIN32GUI
-	int  count ;
-	Window  parent ;
-	Window  root ;
-	Window *  list ;
-	u_int  n ;
-	int  count ;
-
-	win->pic_mod = pic_mod ;
-
-	if( win->my_window == None)
-	{
-		/*
-		 * If Window is not still created , actual drawing is delayed and
-		 * ReparentNotify event will do transparent processing automatically after
-		 * x_window_show().
-		 */
-
-		win->is_transparent = 1 ;
-
-		goto  end ;
-	}
-
-	if( win->pic_mod || x_root_pixmap_available( win->display))
-	{
-		update_pic_transparent( win) ;
-	}
-	else
-	{
-		set_transparent( win , ParentRelative) ;
-
-		XSetWindowBackgroundPixmap( win->display , win->my_window , ParentRelative) ;
-
-		parent = win->my_window ;
-
-		while( 1)
-		{
-			XQueryTree( win->display , parent , &root , &parent , &list , &n) ;
-			XFree(list) ;
-
-			if( parent == DefaultRootWindow(win->display))
-			{
-				break ;
-			}
-
-			XSetWindowBackgroundPixmap( win->display , parent , ParentRelative) ;
-		}
-
-		if( win->window_exposed)
-		{
-			x_window_clear_all( win) ;
-			(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
-		}
-	}
-	
-end:
-	for( count = 0 ; count < win->num_of_children ; count ++)
-	{
-		x_window_set_transparent( win->children[count] , win->pic_mod) ;
-	}
-
-	return  1 ;
-#endif
 }
 
 int
@@ -1277,52 +1074,6 @@ x_window_unset_transparent(
 	)
 {
 	return  unset_transparent( x_get_root_window( win)) ;
-
-#ifndef  USE_WIN32GUI
-	int  count ;
-
-	if( ! win->is_transparent)
-	{
-		/* already unset */
-
-		return  1 ;
-	}
-
-	unset_transparent( win) ;
-
-	if( win->window_exposed)
-	{
-		x_window_clear_all( win) ;
-		(*win->window_exposed)( win , 0 , 0 , win->width , win->height) ;
-	}
-
-	for( count = 0 ; count < win->num_of_children ; count ++)
-	{
-		x_window_unset_transparent( win->children[count]) ;
-	}
-
-	return  1 ;
-#endif
-}
-
-/*
- * Double buffering.
- */
-int
-x_window_use_buffer(
-	x_window_t *  win
-	)
-{
-#ifndef  USE_WIN32GUI
-	win->use_buffer = 1 ;
-
-	win->event_mask &= ~VisibilityChangeMask ;
-
-	/* always true */
-	win->is_scrollable = 1 ;
-#endif
-
-	return  1 ;
 }
 
 int
@@ -1342,7 +1093,12 @@ x_window_set_fg_color(
 	x_color_t *  fg_color
 	)
 {
-	win->fg_color = fg_color->pixel ;
+	if( win->fg_color.pixel == fg_color->pixel)
+	{
+		return  0 ;
+	}
+
+	win->fg_color = *fg_color ;
 
 	return  1 ;
 }
@@ -1353,7 +1109,12 @@ x_window_set_bg_color(
 	x_color_t *  bg_color
 	)
 {
-	win->bg_color = bg_color->pixel ;
+	if( win->bg_color.pixel == bg_color->pixel)
+	{
+		return  0 ;
+	}
+
+	win->bg_color = *bg_color ;
 
 	if( win->my_window != None)
 	{
@@ -1420,12 +1181,12 @@ x_window_get_fg_gc(
 	}
 	
 #if  0
-	x_gc_set_fg_color( win->gc , win->fg_color) ;
-	x_gc_set_bg_color( win->gc , win->bg_color) ;
+	x_gc_set_fg_color( win->gc , win->fg_color.pixel) ;
+	x_gc_set_bg_color( win->gc , win->bg_color.pixel) ;
 #endif
 
-	x_release_pen( x_gc_set_pen( win->gc , x_acquire_pen( win->fg_color))) ;
-	x_release_brush( x_gc_set_brush( win->gc , x_acquire_brush( win->fg_color))) ;
+	x_release_pen( x_gc_set_pen( win->gc , x_acquire_pen( win->fg_color.pixel))) ;
+	x_release_brush( x_gc_set_brush( win->gc , x_acquire_brush( win->fg_color.pixel))) ;
 
 	return  win->gc->gc ;
 }
@@ -1441,12 +1202,12 @@ x_window_get_bg_gc(
 	}
 	
 #if  0
-	x_gc_set_fg_color( win->gc , win->bg_color) ;
-	x_gc_set_bg_color( win->gc , win->fg_color) ;
+	x_gc_set_fg_color( win->gc , win->bg_color.pixel) ;
+	x_gc_set_bg_color( win->gc , win->fg_color.pixel) ;
 #endif
 
 	x_release_pen( x_gc_set_pen( win->gc , GetStockObject(NULL_PEN))) ;
-	x_release_brush( x_gc_set_brush( win->gc , x_acquire_brush( win->bg_color))) ;
+	x_release_brush( x_gc_set_brush( win->gc , x_acquire_brush( win->bg_color.pixel))) ;
 
 	return  win->gc->gc ;
 }
@@ -1494,16 +1255,21 @@ x_window_show(
 #endif
 
 #ifndef  UTF16_IME_CHAR
-	win->my_window = CreateWindowEx( 0 , "MLTERM" , "mlterm" ,
+	win->my_window = CreateWindowEx( 0 , "MLTERM" , win->app_name ,
 #else
-	win->my_window = CreateWindowExW( 0 , L"MLTERM" , L"mlterm" ,
+	win->my_window = CreateWindowExW( 0 , L"MLTERM" , win->app_name ,
 #endif
-			win->parent_window ? WS_CHILD | WS_VISIBLE : WS_OVERLAPPEDWINDOW ,
-			win->parent_window ? win->x : CW_USEDEFAULT ,
-			win->parent_window ? win->y : CW_USEDEFAULT ,
-        		win->parent_window ? ACTUAL_WIDTH(win) : ACTUAL_WINDOW_WIDTH(win) ,
-			win->parent_window ? ACTUAL_HEIGHT(win) : ACTUAL_WINDOW_HEIGHT(win) ,
-			win->parent_window , NULL , win->disp->display->hinst , NULL) ;
+				PARENT_WINDOWID_IS_TOP(win) ?
+					WS_OVERLAPPEDWINDOW : WS_CHILD | WS_VISIBLE ,
+				PARENT_WINDOWID_IS_TOP(win) ?
+					CW_USEDEFAULT : win->x ,
+				PARENT_WINDOWID_IS_TOP(win) ?
+					CW_USEDEFAULT : win->y ,
+				PARENT_WINDOWID_IS_TOP(win) ?
+					ACTUAL_WINDOW_WIDTH(win) : ACTUAL_WIDTH(win) ,
+				PARENT_WINDOWID_IS_TOP(win) ?
+					ACTUAL_WINDOW_HEIGHT(win) : ACTUAL_HEIGHT(win) ,
+				win->parent_window , NULL , win->disp->display->hinst , NULL) ;
 
   	if( ! win->my_window)
         {
@@ -1512,15 +1278,6 @@ x_window_show(
           	return  0 ;
         }
 
-#ifndef  USE_WIN32GUI
-	if( win->use_buffer)
-	{
-		win->drawable = win->buffer = XCreatePixmap( win->display , win->parent_window ,
-				ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
-				DefaultDepth( win->display , win->screen)) ;
-	}
-#endif
-
 	/*
 	 * This should be called after Window Manager settings, because
 	 * x_set_{window|icon}_name() can be called in win->window_realized().
@@ -1611,17 +1368,6 @@ x_window_resize(
 	win->width = width ;
 	win->height = height ;
 
-#ifndef  USE_WIN32GUI
-	if( win->buffer)
-	{
-		XFreePixmap( win->display , win->buffer) ;
-		win->buffer = XCreatePixmap( win->display ,
-			win->parent_window , ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
-			DefaultDepth( win->display , win->screen)) ;
-		win->drawable = win->buffer ;
-	}
-#endif
-
 	if( (flag & NOTIFY_TO_PARENT) && win->parent && win->parent->child_window_resized)
 	{
 		(*win->parent->child_window_resized)( win->parent , win) ;
@@ -1667,11 +1413,6 @@ x_window_set_normal_hints(
 	u_int  height_inc
 	)
 {
-#ifndef  USE_WIN32GUI
-	XSizeHints  size_hints ;
-	x_window_t *  root ;
-#endif
-
 	win->min_width = min_width ;
 	win->min_height = min_height  ;
 	win->base_width = base_width ;
@@ -1679,23 +1420,6 @@ x_window_set_normal_hints(
 	win->width_inc = width_inc ;
 	win->height_inc = height_inc ;
 
-#ifndef  USE_WIN32GUI
-	root = x_get_root_window(win) ;
-
-	/*
-	 * these hints must be set at the same time !
-	 */
-	size_hints.width_inc = total_width_inc( root) ;
-	size_hints.height_inc = total_height_inc( root) ;
-	size_hints.min_width = total_min_width( root) ;
-	size_hints.min_height = total_min_height( root) ;
-	size_hints.base_width = total_base_width( root) ;
-	size_hints.base_height = total_base_height( root) ;
-	size_hints.flags = PMinSize | PResizeInc | PBaseSize ;
-
-	XSetWMNormalHints( root->display , root->my_window , &size_hints) ;
-#endif
-
 	return  1 ;
 }
 
@@ -1761,7 +1485,8 @@ x_window_clear(
 	else
 	{
 		x_release_pen( x_gc_set_pen( win->gc, GetStockObject(NULL_PEN))) ;
-		x_release_brush( x_gc_set_brush( win->gc, x_acquire_brush( win->bg_color))) ;
+		x_release_brush( x_gc_set_brush( win->gc,
+			x_acquire_brush( win->bg_color.pixel))) ;
 		
 		Rectangle( win->gc->gc, r.left, r.top, r.right, r.bottom) ;
 
@@ -1785,7 +1510,8 @@ x_window_clear_margin_area(
 	else
 	{
 		x_release_pen( x_gc_set_pen( win->gc, GetStockObject(NULL_PEN))) ;
-		x_release_brush( x_gc_set_brush( win->gc, x_acquire_brush( win->bg_color))) ;
+		x_release_brush( x_gc_set_brush( win->gc,
+			x_acquire_brush( win->bg_color.pixel))) ;
 
 		Rectangle( win->gc->gc, 0, 0, win->margin, ACTUAL_HEIGHT(win)) ;
 		Rectangle( win->gc->gc, win->margin, 0, win->width + win->margin, win->margin) ;
@@ -1820,7 +1546,7 @@ x_window_fill(
 		return  0 ;
 	}
 	
-	x_release_pen( x_gc_set_pen( win->gc, x_acquire_pen( win->fg_color))) ;
+	x_release_pen( x_gc_set_pen( win->gc , x_acquire_pen( win->fg_color.pixel))) ;
 
 	if( height == 1)
 	{
@@ -1829,7 +1555,8 @@ x_window_fill(
 	}
 	else
 	{
-		x_release_brush( x_gc_set_brush( win->gc, x_acquire_brush( win->fg_color))) ;
+		x_release_brush( x_gc_set_brush( win->gc ,
+			x_acquire_brush( win->fg_color.pixel))) ;
 
 		Rectangle( win->gc->gc, win->margin + x, win->margin + y,
 					win->margin + x + width, win->margin + y + height) ;
@@ -2105,7 +1832,7 @@ x_window_receive_event(
 			if( win->parent != NULL)	/* XXX Hack for not flashing. */
 			{
 				old = SelectObject( (HDC)event->wparam,
-						x_acquire_brush( win->bg_color)) ;
+						x_acquire_brush( win->bg_color.pixel)) ;
 				GetClientRect( win->my_window, &rt) ;
 				PatBlt( (HDC)event->wparam, rt.left, rt.top,
 					rt.right-rt.left, rt.bottom-rt.top, PATCOPY) ;
@@ -2412,7 +2139,7 @@ x_window_receive_event(
 		return  1 ;
 
 	case  WM_MOVE:
-		if( ! win->use_buffer && win->parent == NULL)
+		if( win->parent == NULL)
 		{
 			win->x = LOWORD(event->lparam) ;
 			win->y = HIWORD(event->lparam) ;
@@ -2497,22 +2224,6 @@ x_window_receive_event(
 					win->width = width - win->margin * 2 ;
 					win->height = height - win->margin * 2 ;
 
-				#ifndef  USE_WIN32GUI
-					if( win->buffer)
-					{
-						XFreePixmap( win->display , win->buffer) ;
-						win->buffer = XCreatePixmap( win->display ,
-							win->parent_window ,
-							ACTUAL_WIDTH(win) , ACTUAL_HEIGHT(win) ,
-							DefaultDepth( win->display , win->screen)) ;
-						win->drawable = win->buffer ;
-
-					#ifdef  USE_TYPE_XFT
-						XftDrawChange( win->xft_draw , win->drawable) ;
-					#endif
-					}
-				#endif
-
 					x_window_clear_all( win) ;
 
 					(*win->window_resized)( win) ;
@@ -2669,13 +2380,14 @@ x_window_scroll_leftward_region(
 	}
 
 #ifndef  USE_WIN32GUI
-	XCopyArea( win->display , win->drawable , win->drawable , win->gc ,
+	XCopyArea( win->display , win->my_window , win->my_window , win->gc ,
 		win->margin + boundary_start + width , win->margin ,	/* src */
 		boundary_end - boundary_start - width , win->height ,	/* size */
 		win->margin + boundary_start , win->margin) ;		/* dst */
 
 	return  1 ;
 #else
+	/* XXX Not implemented yet. */
 	return  0 ;
 #endif
 }
@@ -2713,13 +2425,14 @@ x_window_scroll_rightward_region(
 	}
 
 #ifndef  USE_WIN32GUI
-	XCopyArea( win->display , win->drawable , win->drawable , win->gc ,
+	XCopyArea( win->display , win->my_window , win->my_window , win->gc ,
 		win->margin + boundary_start , win->margin ,
 		boundary_end - boundary_start - width , win->height ,
 		win->margin + boundary_start + width , win->margin) ;
 
 	return  1 ;
 #else
+	/* XXX Not implemented yet. */
 	return  0 ;
 #endif
 }
@@ -2762,13 +2475,13 @@ x_window_draw_decsp_image_string(
 		{
 			XSetBackground( win->display , win->ch_gc , bg_color->pixel) ;
 			x_decsp_font_draw_image_string( font->decsp_font ,
-						  win->display , win->drawable , win->ch_gc ,
+						  win->display , win->my_window , win->ch_gc ,
 						  x + win->margin , y + win->margin , str , len) ;
 		}
 		else
 		{
 			x_decsp_font_draw_string( font->decsp_font ,
-						  win->display , win->drawable , win->ch_gc ,
+						  win->display , win->my_window , win->ch_gc ,
 						  x + win->margin , y + win->margin , str , len) ;
 		}
 		return  1 ;
@@ -2958,7 +2671,7 @@ x_window_draw_rect_frame(
 		return  0 ;
 	}
 
-	x_release_pen( x_gc_set_pen( win->gc, x_acquire_pen( win->fg_color))) ;
+	x_release_pen( x_gc_set_pen( win->gc, x_acquire_pen( win->fg_color.pixel))) ;
 	x_release_brush( x_gc_set_brush( win->gc, GetStockObject(NULL_BRUSH))) ;
 	
 	Rectangle( win->gc->gc, x1, y1, x2, y2) ;
@@ -2980,7 +2693,7 @@ x_window_draw_line(
 		return  0 ;
 	}
 
-	x_release_pen( x_gc_set_pen( win->gc, x_acquire_pen( win->fg_color))) ;
+	x_release_pen( x_gc_set_pen( win->gc, x_acquire_pen( win->fg_color.pixel))) ;
 
 	MoveToEx( win->gc->gc, x1, y1, NULL) ;
 	LineTo( win->gc->gc, x2, y2) ;
@@ -3152,9 +2865,7 @@ x_set_window_name(
 	u_char *  name
 	)
 {
-#ifndef  USE_WIN32GUI
 	x_window_t *  root ;
-	XTextProperty  prop ;
 
 	root = x_get_root_window( win) ;
 
@@ -3163,19 +2874,12 @@ x_set_window_name(
 		name = root->app_name ;
 	}
 
-	if( XmbTextListToTextProperty( root->display , (char**)&name , 1 , XStdICCTextStyle , &prop)
-		>= Success)
-	{
-		XSetWMName( root->display , root->my_window , &prop) ;
-
-		XFree( prop.value) ;
-	}
-	else
-	{
-		/* XXX which is better , doing this or return 0 without doing anything ? */
-		XStoreName( root->display , root->my_window , name) ;
-	}
-#endif
+	/*
+	 * XXX
+	 * Convert name to current locale encoding or UTF16.
+	 * Use SetWindowTextW if UTF16_IME_CHAR is defined.
+	 */
+	SetWindowText( root->my_window , name) ;
 
 	return  1 ;
 }
@@ -3356,7 +3060,7 @@ x_window_get_mod_meta_mask(
 	char *  mod_key
 	)
 {
-	return  0 ;
+	return  ModMask ;
 }
 
 int
@@ -3427,7 +3131,7 @@ x_window_paste(
 		kik_warn_printf( KIK_DEBUG_TAG " height is modified -> %d\n" , src_height) ;
 	}
 
-	XCopyArea( win->display , src , win->drawable , win->gc ,
+	XCopyArea( win->display , src , win->my_window , win->gc ,
 		src_x , src_y , src_width , src_height ,
 		dst_x + win->margin , dst_y + win->margin) ;
 
